@@ -6,7 +6,7 @@
  * Copyright (C) 1999
  *  David Corcoran <corcoran@linuxnet.com>
  *
- * $Id: configfile.h 1550 2005-06-15 11:54:21Z rousseau $
+ * $Id: configfile.h 2331 2007-01-11 16:54:51Z rousseau $
  */
 
 #ifndef __configfile_h__
@@ -17,7 +17,7 @@ extern "C"
 {
 #endif
 
-	int DBGetReaderList(char *readerconf, SerialReader **caller_reader_list);
+	int DBGetReaderList(const char *readerconf, SerialReader **caller_reader_list);
 
 #ifdef __cplusplus
 }
