@@ -7,7 +7,7 @@
  *  Toni Andjelkovic <toni@soth.at>
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *
- * $Id: hotplug_libusb.c,v 1.1.1.9 2006-08-11 18:49:09 rousseau Exp $
+ * $Id: hotplug_libusb.c,v 1.8 2007-01-25 14:55:37 rousseau Exp $
  */
 
 /**
@@ -55,7 +55,10 @@ extern PCSCLITE_MUTEX usbNotifierMutex;
 static PCSCLITE_THREAD_T usbNotifyThread;
 static int driverSize = -1;
 static char AraKiriHotPlug = FALSE;
-char ReCheckSerialReaders = FALSE;
+extern int HPForceReaderPolling;
+
+/* values of ifdCapabilities bits */
+#define IFD_GENERATE_HOTPLUG 1
 
 /*
  * keep track of drivers in a dynamically allocated array
@@ -68,6 +71,7 @@ static struct _driverTracker
 	char *bundleName;
 	char *libraryPath;
 	char *readerName;
+	int ifdCapabilities;
 } *driverTracker = NULL;
 #define DRIVER_TRACKER_SIZE_STEP 8
 
@@ -79,14 +83,14 @@ static struct _readerTracker
 	char status;
 	char bus_device[BUS_DEVICE_STRSIZE];	/* device name */
 	char *fullName;	/* full reader name (including serial number) */
-
-	struct _driverTracker *driver;	/* driver for this reader */
 } readerTracker[PCSCLITE_MAX_READERS_CONTEXTS];
 
 LONG HPReadBundleValues(void);
 LONG HPAddHotPluggable(struct usb_device *dev, const char bus_device[],
 	struct _driverTracker *driver);
 LONG HPRemoveHotPluggable(int index);
+static void HPRescanUsbBus(void);
+static void HPEstablishUSBNotifications(void);
 
 LONG HPReadBundleValues(void)
 {
@@ -167,6 +171,12 @@ LONG HPReadBundleValues(void)
 					driverTracker[listCount].libraryPath = strdup(fullLibPath);
 				}
 
+				/* Get ifdCapabilities */
+				rv = LTPBundleFindValueWithKey(fullPath, PCSCLITE_HP_CPCTKEY_NAME,
+					keyValue, 0);
+				if (rv == 0)
+					driverTracker[listCount].ifdCapabilities = strtol(keyValue, 0, 16);
+
 #ifdef DEBUG_HOTPLUG
 					Log2(PCSC_LOG_INFO, "Found driver for: %s",
 						driverTracker[listCount].readerName);
@@ -202,6 +212,7 @@ LONG HPReadBundleValues(void)
 						driverTracker[i].bundleName = NULL;
 						driverTracker[i].libraryPath = NULL;
 						driverTracker[i].readerName = NULL;
+						driverTracker[i].ifdCapabilities = 0;
 					}
 				}
 			}
@@ -226,149 +237,177 @@ LONG HPReadBundleValues(void)
 	return rv;
 }
 
-void HPEstablishUSBNotifications(void)
+void HPRescanUsbBus(void)
 {
 	int i, j;
 	struct usb_bus *bus;
 	struct usb_device *dev;
 	char bus_device[BUS_DEVICE_STRSIZE];
 
-	usb_init();
-	while (1)
+	usb_find_busses();
+	usb_find_devices();
+
+	for (i=0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
+		/* clear rollcall */
+		readerTracker[i].status = READER_ABSENT;
+
+	/* For each USB bus */
+	for (bus = usb_get_busses(); bus; bus = bus->next)
 	{
-		usb_find_busses();
-		usb_find_devices();
-
-		for (i=0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
-			/* clear rollcall */
-			readerTracker[i].status = READER_ABSENT;
-
-		/* For each USB bus */
-		for (bus = usb_get_busses(); bus; bus = bus->next)
+		/* For each USB device */
+		for (dev = bus->devices; dev; dev = dev->next)
 		{
-			/* For each USB device */
-			for (dev = bus->devices; dev; dev = dev->next)
-			{
-				/* check if the device is supported by one driver */
-				for (i=0; i<driverSize; i++)
-				{
-					if (driverTracker[i].libraryPath != NULL &&
-						dev->descriptor.idVendor == driverTracker[i].manuID &&
-						dev->descriptor.idProduct == driverTracker[i].productID)
-					{
-						int newreader;
-
-						/* A known device has been found */
-						snprintf(bus_device, BUS_DEVICE_STRSIZE, "%s:%s",
-							bus->dirname, dev->filename);
-						bus_device[BUS_DEVICE_STRSIZE - 1] = '\0';
-#ifdef DEBUG_HOTPLUG
-						Log2(PCSC_LOG_DEBUG, "Found matching USB device: %s", bus_device);
-#endif
-						newreader = TRUE;
-
-						/* Check if the reader is a new one */
-						for (j=0; j<PCSCLITE_MAX_READERS_CONTEXTS; j++)
-						{
-							if (strncmp(readerTracker[j].bus_device,
-								bus_device, BUS_DEVICE_STRSIZE) == 0)
-							{
-								/* The reader is already known */
-								readerTracker[j].status = READER_PRESENT;
-								newreader = FALSE;
-#ifdef DEBUG_HOTPLUG
-								Log2(PCSC_LOG_DEBUG, "Refresh USB device: %s", bus_device);
-#endif
-								break;
-							}
-						}
-
-						/* New reader found */
-						if (newreader)
-							HPAddHotPluggable(dev, bus_device, &driverTracker[i]);
-					}
-				}
-			} /* End of USB device for..loop */
-
-		} /* End of USB bus for..loop */
-
-		/*
-		 * check if all the previously found readers are still present
-		 */
-		for (i=0; i<PCSCLITE_MAX_READERS_CONTEXTS; i++)
-		{
-#if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)
-			int fd;
-			char filename[BUS_DEVICE_STRSIZE];
-
-			/*	BSD workaround:
-			 *	ugenopen() in sys/dev/usb/ugen.c returns EBUSY
-			 *	when the character device file is already open.
-			 *	Because of this, open usb devices will not be
-			 *	detected by usb_find_devices(), so we have to
-			 *	check for this explicitly.
-			 */
-			if (readerTracker[i].status == READER_PRESENT ||
-				 readerTracker[i].driver == NULL)
-				continue;
-
-			sscanf(readerTracker[i].bus_device, "%*[^:]%*[:]%s", filename);
-			fd = open(filename, O_RDONLY);
-			if (fd == -1)
-			{
-				if (errno == EBUSY)
-				{
-					/* The device is present */
-#ifdef DEBUG_HOTPLUG
-					Log2(PCSC_LOG_DEBUG, "BSD: EBUSY on %s", filename);
-#endif
-					readerTracker[i].status = READER_PRESENT;
-				}
-#ifdef DEBUG_HOTPLUG
-				else
-					Log3(PCSC_LOG_DEBUG, "BSD: %s error: %s", filename,
-						strerror(errno));
-#endif
-			}
-			else
-			{
-#ifdef DEBUG_HOTPLUG
-				Log2(PCSC_LOG_DEBUG, "BSD: %s still present", filename);
-#endif
-				readerTracker[i].status = READER_PRESENT;
-				close(fd);
-			}
-#endif
-			if (readerTracker[i].status == READER_ABSENT &&
-					readerTracker[i].driver != NULL)
-				HPRemoveHotPluggable(i);
-		}
-
-		SYS_Sleep(1);
-		if (AraKiriHotPlug)
-		{
-			int retval;
-
+			/* check if the device is supported by one driver */
 			for (i=0; i<driverSize; i++)
 			{
-				/* free strings allocated by strdup() */
-				free(driverTracker[i].bundleName);
-				free(driverTracker[i].libraryPath);
-				free(driverTracker[i].readerName);
+				if (driverTracker[i].libraryPath != NULL &&
+					dev->descriptor.idVendor == driverTracker[i].manuID &&
+					dev->descriptor.idProduct == driverTracker[i].productID)
+				{
+					int newreader;
+
+					/* A known device has been found */
+					snprintf(bus_device, BUS_DEVICE_STRSIZE, "%s:%s",
+						bus->dirname, dev->filename);
+					bus_device[BUS_DEVICE_STRSIZE - 1] = '\0';
+#ifdef DEBUG_HOTPLUG
+					Log2(PCSC_LOG_DEBUG, "Found matching USB device: %s", bus_device);
+#endif
+					newreader = TRUE;
+
+					/* Check if the reader is a new one */
+					for (j=0; j<PCSCLITE_MAX_READERS_CONTEXTS; j++)
+					{
+						if (strncmp(readerTracker[j].bus_device,
+							bus_device, BUS_DEVICE_STRSIZE) == 0)
+						{
+							/* The reader is already known */
+							readerTracker[j].status = READER_PRESENT;
+							newreader = FALSE;
+#ifdef DEBUG_HOTPLUG
+							Log2(PCSC_LOG_DEBUG, "Refresh USB device: %s", bus_device);
+#endif
+							break;
+						}
+					}
+
+					/* New reader found */
+					if (newreader)
+						HPAddHotPluggable(dev, bus_device, &driverTracker[i]);
+				}
 			}
-			free(driverTracker);
+		} /* End of USB device for..loop */
 
-			Log1(PCSC_LOG_INFO, "Hotplug stopped");
-			pthread_exit(&retval);
-		}
+	} /* End of USB bus for..loop */
 
-		if (ReCheckSerialReaders)
+	/*
+	 * check if all the previously found readers are still present
+	 */
+	for (i=0; i<PCSCLITE_MAX_READERS_CONTEXTS; i++)
+	{
+#if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)
+		int fd;
+		char filename[BUS_DEVICE_STRSIZE];
+
+		/*	BSD workaround:
+		 *	ugenopen() in sys/dev/usb/ugen.c returns EBUSY
+		 *	when the character device file is already open.
+		 *	Because of this, open usb devices will not be
+		 *	detected by usb_find_devices(), so we have to
+		 *	check for this explicitly.
+		 */
+		if (readerTracker[i].status == READER_PRESENT ||
+			 readerTracker[i].fullName == NULL)
+			continue;
+
+		sscanf(readerTracker[i].bus_device, "%*[^:]%*[:]%s", filename);
+		fd = open(filename, O_RDONLY);
+		if (fd == -1)
 		{
-			ReCheckSerialReaders = FALSE;
-			RFReCheckReaderConf();
+			if (errno == EBUSY)
+			{
+				/* The device is present */
+#ifdef DEBUG_HOTPLUG
+				Log2(PCSC_LOG_DEBUG, "BSD: EBUSY on %s", filename);
+#endif
+				readerTracker[i].status = READER_PRESENT;
+			}
+#ifdef DEBUG_HOTPLUG
+			else
+				Log3(PCSC_LOG_DEBUG, "BSD: %s error: %s", filename,
+					strerror(errno));
+#endif
 		}
+		else
+		{
+#ifdef DEBUG_HOTPLUG
+			Log2(PCSC_LOG_DEBUG, "BSD: %s still present", filename);
+#endif
+			readerTracker[i].status = READER_PRESENT;
+			close(fd);
+		}
+#endif
+		if ((readerTracker[i].status == READER_ABSENT) &&
+			(readerTracker[i].fullName != NULL))
+			HPRemoveHotPluggable(i);
+	}
 
-	}	/* End of while loop */
+	if (AraKiriHotPlug)
+	{
+		int retval;
+
+		for (i=0; i<driverSize; i++)
+		{
+			/* free strings allocated by strdup() */
+			free(driverTracker[i].bundleName);
+			free(driverTracker[i].libraryPath);
+			free(driverTracker[i].readerName);
+		}
+		free(driverTracker);
+
+		Log1(PCSC_LOG_INFO, "Hotplug stopped");
+		pthread_exit(&retval);
+	}
+}
+
+void HPEstablishUSBNotifications(void)
+{
+	int i, do_polling;
+
+	/* libusb default is /dev/bus/usb but the devices are not yet visible there
+	 * when a hotplug is requested */
+	setenv("USB_DEVFS_PATH", "/proc/bus/usb", 0);
+
+	usb_init();
+
+	/* scan the USB bus for devices at startup */
+	HPRescanUsbBus();
+
+	/* if at least one driver do not have IFD_GENERATE_HOTPLUG */
+	do_polling = FALSE;
+	for (i=0; i<driverSize; i++)
+		if (driverTracker[i].libraryPath)
+			if ((driverTracker[i].ifdCapabilities & IFD_GENERATE_HOTPLUG) == 0)
+			{
+				Log2(PCSC_LOG_INFO,
+					"Driver %s does not support IFD_GENERATE_HOTPLUG",
+					driverTracker[i].bundleName);
+				do_polling = TRUE;
+				break;
+			}
+
+	if (HPForceReaderPolling)
+	{
+		Log2(PCSC_LOG_INFO,
+				"Polling forced every %d second(s)", HPForceReaderPolling);
+		do_polling = TRUE;
+	}
+
+	while (do_polling)
+	{
+		SYS_Sleep(HPForceReaderPolling);
+		HPRescanUsbBus();
+	}
 }
 
 LONG HPSearchHotPluggables(void)
@@ -377,9 +416,9 @@ LONG HPSearchHotPluggables(void)
 
 	for (i=0; i<PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		readerTracker[i].driver = NULL;
 		readerTracker[i].status = READER_ABSENT;
 		readerTracker[i].bus_device[0] = '\0';
+		readerTracker[i].fullName = NULL;
 	}
 
 	if (HPReadBundleValues())
@@ -413,7 +452,7 @@ LONG HPAddHotPluggable(struct usb_device *dev, const char bus_device[],
 	/* find a free entry */
 	for (i=0; i<PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if (readerTracker[i].driver == NULL)
+		if (readerTracker[i].fullName == NULL)
 			break;
 	}
 
@@ -427,8 +466,6 @@ LONG HPAddHotPluggable(struct usb_device *dev, const char bus_device[],
 	strncpy(readerTracker[i].bus_device, bus_device,
 		sizeof(readerTracker[i].bus_device));
 	readerTracker[i].bus_device[sizeof(readerTracker[i].bus_device) - 1] = '\0';
-   
-	readerTracker[i].driver = driver;
 
 #ifdef ADD_SERIAL_NUMBER
 	if (dev->descriptor.iSerialNumber)
@@ -473,7 +510,7 @@ LONG HPRemoveHotPluggable(int index)
 	free(readerTracker[index].fullName);
 	readerTracker[index].status = READER_ABSENT;
 	readerTracker[index].bus_device[0] = '\0';
-	readerTracker[index].driver = NULL;
+	readerTracker[index].fullName = NULL;
 
 	SYS_MutexUnLock(&usbNotifierMutex);
 
@@ -490,7 +527,8 @@ ULONG HPRegisterForHotplugEvents(void)
 
 void HPReCheckSerialReaders(void)
 {
-	ReCheckSerialReaders = TRUE;
+	HPRescanUsbBus();
+	RFReCheckReaderConf();
 }
 
 #endif
