@@ -5,7 +5,7 @@
  *  Toni Andjelkovic <toni@soth.at>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: parser.h,v 1.1.1.5 2006-04-22 18:58:39 rousseau Exp $
+ * $Id: parser.h,v 1.7 2007-01-25 14:55:37 rousseau Exp $
  */
 
 /**
@@ -27,7 +27,7 @@ extern "C"
 #define TOKEN_TYPE_KEY         1
 #define TOKEN_TYPE_STRING      2
 
-int LTPBundleFindValueWithKey(char *fileName, char *tokenKey,
+int LTPBundleFindValueWithKey(const char *fileName, const char *tokenKey,
                               char *tokenValue, int tokenIndice);
 
 #ifdef __cplusplus
