@@ -5,7 +5,7 @@
  *  David Corcoran <corcoran@linuxnet.com>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: pcscdaemon.c 1827 2006-01-24 14:49:52Z rousseau $
+ * $Id: pcscdaemon.c 2335 2007-01-11 18:05:12Z rousseau $
  */
 
 /**
@@ -51,10 +51,10 @@
 
 char AraKiri = FALSE;
 static char Init = TRUE;
-extern char ReCheckSerialReaders;
+int HPForceReaderPolling = 0;
 
 /*
- * Some internal functions 
+ * Some internal functions
  */
 void SVCServiceRunLoop(void);
 void SVCClientCleanup(psharedSegmentMsg);
@@ -67,13 +67,66 @@ void print_usage (char const * const);
 
 PCSCLITE_MUTEX usbNotifierMutex;
 
+#ifdef USE_RUN_PID
+pid_t GetDaemonPid(void);
+pid_t GetDaemonPid(void)
+{
+	FILE *f;
+	pid_t pid;
+
+	/* pids are only 15 bits but 4294967296
+	 * (32 bits in case of a new system use it) is on 10 bytes
+	 */
+	if ((f = fopen(USE_RUN_PID, "rb")) != NULL)
+	{
+#define PID_ASCII_SIZE 11
+		char pid_ascii[PID_ASCII_SIZE];
+
+		fgets(pid_ascii, PID_ASCII_SIZE, f);
+		fclose(f);
+
+		pid = atoi(pid_ascii);
+	}
+	else
+	{
+		Log2(PCSC_LOG_CRITICAL, "Can't open " USE_RUN_PID ": %s",
+			strerror(errno));
+		return -1;
+	}
+
+	return pid;
+} /* GetDaemonPid */
+#endif
+
+int SendHotplugSignal(void)
+{
+#ifdef USE_RUN_PID
+	pid_t pid;
+
+	pid = GetDaemonPid();
+
+	if (pid != -1)
+	{
+		Log2(PCSC_LOG_INFO, "Send hotplug signal to pcscd (pid=%d)", pid);
+		if (kill(pid, SIGUSR1) < 0)
+		{
+			Log3(PCSC_LOG_CRITICAL, "Can't signal pcscd (pid=%d): %s",
+				pid, strerror(errno));
+			return EXIT_FAILURE ;
+		}
+	}
+#endif
+
+	return EXIT_SUCCESS;
+} /* SendHotplugSignal */
+
 /*
- * Cleans up messages still on the queue when a client dies 
+ * Cleans up messages still on the queue when a client dies
  */
 void SVCClientCleanup(psharedSegmentMsg msgStruct)
 {
 	/*
-	 * May be implemented in future releases 
+	 * May be implemented in future releases
 	 */
 }
 
@@ -82,7 +135,7 @@ void SVCClientCleanup(psharedSegmentMsg msgStruct)
  *
  * An endless loop calls the function \c SHMProcessEventsServer() to check for
  * messages sent by clients.
- * If the message is valid, \c CreateContextThread() is called to serve this 
+ * If the message is valid, \c CreateContextThread() is called to serve this
  * request.
  */
 void SVCServiceRunLoop(void)
@@ -90,12 +143,12 @@ void SVCServiceRunLoop(void)
 	int rsp;
 	LONG rv;
 	DWORD dwClientID;	/* Connection ID used to reference the Client */
-	
+
 	rsp = 0;
 	rv = 0;
 
 	/*
-	 * Initialize the comm structure 
+	 * Initialize the comm structure
 	 */
 	rsp = SHMInitializeCommonSegment();
 
@@ -106,7 +159,7 @@ void SVCServiceRunLoop(void)
 	}
 
 	/*
-	 * Initialize the contexts structure 
+	 * Initialize the contexts structure
 	 */
 	rv = ContextsInitialize();
 
@@ -117,7 +170,7 @@ void SVCServiceRunLoop(void)
 	}
 
 	/*
-	 * Solaris sends a SIGALRM and it is annoying 
+	 * Solaris sends a SIGALRM and it is annoying
 	 */
 
 	signal(SIGALRM, SIG_IGN);
@@ -126,12 +179,12 @@ void SVCServiceRunLoop(void)
 				 * when the shell is existed */
 
 	/*
-	 * This function always returns zero 
+	 * This function always returns zero
 	 */
 	rsp = SYS_MutexInit(&usbNotifierMutex);
 
 	/*
-	 * Set up the search for USB/PCMCIA devices 
+	 * Set up the search for USB/PCMCIA devices
 	 */
 	HPSearchHotPluggables();
 	HPRegisterForHotplugEvents();
@@ -143,7 +196,6 @@ void SVCServiceRunLoop(void)
 
 	while (TRUE)
 	{
-
 		switch (rsp = SHMProcessEventsServer(&dwClientID, 0))
 		{
 
@@ -159,18 +211,14 @@ void SVCServiceRunLoop(void)
 
 			break;
 
-		case 2:
-			/*
-			 * timeout in SHMProcessEventsServer(): do nothing
-			 * this is used to catch the Ctrl-C signal at some time when
-			 * nothing else happens
-			 */
+		case -1:
+			Log1(PCSC_LOG_ERROR, "Error in SHMProcessEventsServer");
 			break;
 
-		case -1:
-			/* do not display if we are exiting or re-reading the config */
-			if ((!AraKiri) && (!ReCheckSerialReaders))
-				Log1(PCSC_LOG_ERROR, "Error in SHMProcessEventsServer");
+		case -2:
+			/* Nothing to do in case of a syscall interrupted
+			 * It happens when SIGUSR1 (reload) or SIGINT (Ctrl-C) is received
+			 * We just try again */
 			break;
 
 		default:
@@ -195,6 +243,7 @@ int main(int argc, char **argv)
 {
 	int rv;
 	char setToForeground;
+	char HotPlug;
 	char *newReaderConfig;
 	struct stat fStatBuf;
 	int opt;
@@ -210,16 +259,20 @@ int main(int argc, char **argv)
 		{"info", 0, 0, 0},
 		{"error", 0, 0, 'e'},
 		{"critical", 0, 0, 'C'},
+		{"hotplug", 0, 0, 'H'},
+		{"force-reader-polling", optional_argument, 0, 0},
 		{0, 0, 0, 0}
 	};
 #endif
-	
+#define OPT_STRING "c:fdhvaeCH"
+
 	rv = 0;
-	newReaderConfig = 0;
+	newReaderConfig = NULL;
 	setToForeground = FALSE;
+	HotPlug = FALSE;
 
 	/*
-	 * test the version 
+	 * test the version
 	 */
 	if (strcmp(PCSCLITE_VERSION_NUMBER, VERSION) != 0)
 	{
@@ -238,14 +291,21 @@ int main(int argc, char **argv)
 	DebugLogSetLogType(DEBUGLOG_SYSLOG_DEBUG);
 
 	/*
-	 * Handle any command line arguments 
+	 * Handle any command line arguments
 	 */
 #ifdef  HAVE_GETOPT_LONG
-	while ((opt = getopt_long (argc, argv, "c:fdhvaeC", long_options, &option_index)) != -1) {
+	while ((opt = getopt_long (argc, argv, OPT_STRING, long_options, &option_index)) != -1) {
 #else
-	while ((opt = getopt (argc, argv, "c:fdhvaeC")) != -1) {
+	while ((opt = getopt (argc, argv, OPT_STRING)) != -1) {
 #endif
 		switch (opt) {
+#ifdef  HAVE_GETOPT_LONG
+			case 0:
+				if (strcmp(long_options[option_index].name,
+					"force-reader-polling") == 0)
+					HPForceReaderPolling = optarg ? abs(atoi(optarg)) : 1;
+				break;
+#endif
 			case 'c':
 				Log2(PCSC_LOG_INFO, "using new config file: %s", optarg);
 				newReaderConfig = optarg;
@@ -283,13 +343,19 @@ int main(int argc, char **argv)
 				DebugLogSetCategory(DEBUG_CATEGORY_APDU);
 				break;
 
+			case 'H':
+				/* debug to stderr instead of default syslog */
+				DebugLogSetLogType(DEBUGLOG_STDERR_DEBUG);
+				HotPlug = TRUE;
+				break;
+
 			default:
 				print_usage (argv[0]);
 				return EXIT_FAILURE;
 		}
 
 	}
-	
+
 	if (argv[optind])
 	{
 		printf("Unknown option: %s\n\n", argv[optind]);
@@ -306,24 +372,17 @@ int main(int argc, char **argv)
 	if (rv == 0)
 	{
 #ifdef USE_RUN_PID
+		pid_t pid;
 
 		/* read the pid file to get the old pid and test if the old pcscd is
-		 * still running 
+		 * still running
 		 */
-		FILE *f;
-		/* pids are only 15 bits but 4294967296
-		 * (32 bits in case of a new system use it) is on 10 bytes
-		 */
-#define PID_ASCII_SIZE 11
-		char pid_ascii[PID_ASCII_SIZE];
-		int pid;
+		pid = GetDaemonPid();
 
-		if ((f = fopen(USE_RUN_PID, "rb")) != NULL)
+		if (pid != -1)
 		{
-			fgets(pid_ascii, PID_ASCII_SIZE, f);
-			fclose(f);
-
-			pid = atoi(pid_ascii);
+			if (HotPlug)
+				return SendHotplugSignal();
 
 			if (kill(pid, 0) == 0)
 			{
@@ -338,34 +397,44 @@ int main(int argc, char **argv)
 				clean_temp_files();
 		}
 		else
+#endif
 		{
+			if (HotPlug)
+			{
+#ifdef USE_RUN_PID
+				Log1(PCSC_LOG_CRITICAL, "file " USE_RUN_PID " do not exist");
+#else
+				Log1(PCSC_LOG_CRITICAL,
+					"pcscd was not configured with --enable-runpid=FILE");
+#endif
+				Log1(PCSC_LOG_CRITICAL, "Hotplug failed");
+				return EXIT_FAILURE;
+			}
+
 			Log1(PCSC_LOG_CRITICAL,
 				"file " PCSCLITE_PUBSHM_FILE " already exists.");
 			Log1(PCSC_LOG_CRITICAL,
 				"Maybe another pcscd is running?");
-			Log1(PCSC_LOG_CRITICAL, 
+#ifdef USE_RUN_PID
+			Log1(PCSC_LOG_CRITICAL,
 				"I can't read process pid from " USE_RUN_PID);
+#endif
 			Log1(PCSC_LOG_CRITICAL,
 				"Remove " PCSCLITE_PUBSHM_FILE " and " PCSCLITE_CSOCK_NAME);
 			Log1(PCSC_LOG_CRITICAL,
 				"if pcscd is not running to clear this message.");
 			return EXIT_FAILURE;
 		}
-#else
-		Log1(PCSC_LOG_CRITICAL,
-			"file " PCSCLITE_PUBSHM_FILE " already exists.");
-		Log1(PCSC_LOG_CRITICAL,
-			"Maybe another pcscd is running?");
-		Log1(PCSC_LOG_CRITICAL,
-			"Remove " PCSCLITE_PUBSHM_FILE " and " PCSCLITE_CSOCK_NAME);
-		Log1(PCSC_LOG_CRITICAL,
-			"if pcscd is not running to clear this message.");
-		return EXIT_FAILURE;
-#endif
 	}
+	else
+		if (HotPlug)
+		{
+			Log1(PCSC_LOG_CRITICAL, "Hotplug failed: pcscd is not running");
+			return EXIT_FAILURE;
+		}
 
 	/*
-	 * If this is set to one the user has asked it not to fork 
+	 * If this is set to one the user has asked it not to fork
 	 */
 	if (!setToForeground)
 	{
@@ -375,7 +444,7 @@ int main(int argc, char **argv)
 	}
 
 	/*
-	 * cleanly remove /tmp/pcsc when exiting 
+	 * cleanly remove /tmp/pcsc when exiting
 	 */
 	signal(SIGQUIT, signal_trap);
 	signal(SIGTERM, signal_trap);
@@ -418,12 +487,12 @@ int main(int argc, char **argv)
 		Log2(PCSC_LOG_CRITICAL, "atexit() failed: %s", strerror(errno));
 
 	/*
-	 * Allocate memory for reader structures 
+	 * Allocate memory for reader structures
 	 */
 	RFAllocateReaderSpace();
 
 	/*
-	 * Grab the information from the reader.conf 
+	 * Grab the information from the reader.conf
 	 */
 	if (newReaderConfig)
 	{
@@ -445,7 +514,7 @@ int main(int argc, char **argv)
 			Log1(PCSC_LOG_INFO,
 				"warning: no " PCSCLITE_READER_CONFIG " found");
 			/*
-			 * Token error in file 
+			 * Token error in file
 			 */
 		}
 		else
@@ -455,7 +524,7 @@ int main(int argc, char **argv)
 	}
 
 	/*
-	 * Set the default globals 
+	 * Set the default globals
 	 */
 	g_rgSCardT0Pci.dwProtocol = SCARD_PROTOCOL_T0;
 	g_rgSCardT1Pci.dwProtocol = SCARD_PROTOCOL_T1;
@@ -464,12 +533,12 @@ int main(int argc, char **argv)
 	Log1(PCSC_LOG_INFO, "pcsc-lite " VERSION " daemon ready.");
 
 	/*
-	 * post initialistion 
+	 * post initialistion
 	 */
 	Init = FALSE;
 
 	/*
-	 * signal_trap() does just set a global variable used by the main loop 
+	 * signal_trap() does just set a global variable used by the main loop
 	 */
 	signal(SIGQUIT, signal_trap);
 	signal(SIGTERM, signal_trap);
@@ -517,8 +586,24 @@ void clean_temp_files(void)
 
 void signal_reload(int sig)
 {
+	static int rescan_ongoing = FALSE;
+
+	if (AraKiri)
+		return;
+
 	Log1(PCSC_LOG_INFO, "Reload serial configuration");
+	if (rescan_ongoing)
+	{
+		Log1(PCSC_LOG_INFO, "Rescan already ongoing");
+		return;
+	}
+
+	rescan_ongoing = TRUE;
+
 	HPReCheckSerialReaders();
+
+	rescan_ongoing = FALSE;
+	Log1(PCSC_LOG_INFO, "End reload serial configuration");
 } /* signal_reload */
 
 void signal_trap(int sig)
@@ -559,17 +644,20 @@ void print_usage (char const * const progname)
 	printf("  -f, --foreground	run in foreground (no daemon),\n");
 	printf("			send logs to stderr instead of syslog\n");
 	printf("  -h, --help		display usage information\n");
+	printf("  -H, --hotplug		ask the daemon to rescan the avaiable readers\n");
 	printf("  -v, --version		display the program version number\n");
-	printf("  -d, --debug	 	display lower level debug messages\n"); 
-	printf("      --info	 	display info level debug messages (default level)\n"); 
-	printf("  -e  --error	 	display error level debug messages\n"); 
-	printf("  -C  --critical 	display critical only level debug messages\n"); 
+	printf("  -d, --debug	 	display lower level debug messages\n");
+	printf("      --info	 	display info level debug messages (default level)\n");
+	printf("  -e  --error	 	display error level debug messages\n");
+	printf("  -C  --critical 	display critical only level debug messages\n");
+	printf("  --force-reader-polling ignore the IFD_GENERATE_HOTPLUG reader capability\n");
 #else
 	printf("  -a    log APDU commands and results\n");
 	printf("  -c 	path to reader.conf\n");
 	printf("  -f	run in foreground (no daemon), send logs to stderr instead of syslog\n");
-	printf("  -d 	display debug messages. Output may be:\n"); 
+	printf("  -d 	display debug messages. Output may be:\n");
 	printf("  -h 	display usage information\n");
+	printf("  -H	ask the daemon to rescan the avaiable readers\n");
 	printf("  -v 	display the program version number\n");
 #endif
 }
