@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: readerfactory.c 2114 2006-07-31 13:28:50Z rousseau $
+ * $Id: readerfactory.c 2330 2007-01-11 16:54:16Z rousseau $
  */
 
 /**
@@ -53,7 +53,7 @@ LONG RFAllocateReaderSpace(void)
 	int i;   					/* Counter */
 
 	/*
-	 * Allocate each reader structure 
+	 * Allocate each reader structure
 	 */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
@@ -63,7 +63,7 @@ LONG RFAllocateReaderSpace(void)
 	}
 
 	/*
-	 * Create public event structures 
+	 * Create public event structures
 	 */
 	return EHInitializeEventStructures();
 }
@@ -103,7 +103,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	}
 
 	/*
-	 * Same name, same port - duplicate reader cannot be used 
+	 * Same name, same port - duplicate reader cannot be used
 	 */
 	if (dwNumReadersContexts != 0)
 	{
@@ -131,7 +131,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	}
 
 	/*
-	 * We must find an empty slot to put the reader structure 
+	 * We must find an empty slot to put the reader structure
 	 */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
@@ -145,16 +145,18 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	if (i == PCSCLITE_MAX_READERS_CONTEXTS)
 	{
 		/*
-		 * No more spots left return 
+		 * No more spots left return
 		 */
 		return SCARD_E_NO_MEMORY;
 	}
 
 	/*
-	 * Check and set the readername to see if it must be enumerated 
+	 * Check and set the readername to see if it must be enumerated
 	 */
 	parentNode = RFSetReaderName(sReadersContexts[dwContext], lpcReader,
 		lpcLibrary, dwPort, 0);
+	if (parentNode < -1)
+		return SCARD_E_NO_MEMORY;
 
 	strlcpy((sReadersContexts[dwContext])->lpcLibrary, lpcLibrary,
 		sizeof((sReadersContexts[dwContext])->lpcLibrary));
@@ -178,22 +180,22 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		(sReadersContexts[dwContext])->psHandles[i].hCard = 0;
 
 	/*
-	 * If a clone to this reader exists take some values from that clone 
+	 * If a clone to this reader exists take some values from that clone
 	 */
 	if (parentNode >= 0 && parentNode < PCSCLITE_MAX_READERS_CONTEXTS)
 	{
-		(sReadersContexts[dwContext])->pdwFeeds = 
+		(sReadersContexts[dwContext])->pdwFeeds =
 		  (sReadersContexts[parentNode])->pdwFeeds;
 		*(sReadersContexts[dwContext])->pdwFeeds += 1;
-		(sReadersContexts[dwContext])->vHandle = 
+		(sReadersContexts[dwContext])->vHandle =
 		  (sReadersContexts[parentNode])->vHandle;
-		(sReadersContexts[dwContext])->mMutex = 
+		(sReadersContexts[dwContext])->mMutex =
 		  (sReadersContexts[parentNode])->mMutex;
-		(sReadersContexts[dwContext])->pdwMutex = 
+		(sReadersContexts[dwContext])->pdwMutex =
 		  (sReadersContexts[parentNode])->pdwMutex;
 
 		/*
-		 * Call on the driver to see if it is thread safe 
+		 * Call on the driver to see if it is thread safe
 		 */
 		dwGetSize = sizeof(ucThread);
 		rv = IFDGetCapabilities((sReadersContexts[parentNode]),
@@ -213,8 +215,8 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	{
 		(sReadersContexts[dwContext])->pdwFeeds = malloc(sizeof(DWORD));
 
-		/* Initialize pdwFeeds to 1, otherwise multiple 
-		   cloned readers will cause pcscd to crash when 
+		/* Initialize pdwFeeds to 1, otherwise multiple
+		   cloned readers will cause pcscd to crash when
 		   RFUnloadReader unloads the driver library
 		   and there are still devices attached using it --mikeg*/
 
@@ -241,10 +243,10 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	if (rv != SCARD_S_SUCCESS)
 	{
 		/*
-		 * Cannot connect to reader exit gracefully 
+		 * Cannot connect to reader exit gracefully
 		 */
 		/*
-		 * Clean up so it is not using needed space 
+		 * Clean up so it is not using needed space
 		 */
 		Log2(PCSC_LOG_ERROR, "%s init failed.", lpcReader);
 
@@ -255,7 +257,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		(sReadersContexts[dwContext])->dwIdentity = 0;
 
 		/*
-		 * Destroy and free the mutex 
+		 * Destroy and free the mutex
 		 */
 		if (*(sReadersContexts[dwContext])->pdwMutex == 1)
 		{
@@ -289,7 +291,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		return rv;
 
 	/*
-	 * Call on the driver to see if there are multiple slots 
+	 * Call on the driver to see if there are multiple slots
 	 */
 
 	dwGetSize = sizeof(ucGetData);
@@ -299,23 +301,23 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	if (rv != IFD_SUCCESS || dwGetSize != 1 || ucGetData[0] == 0)
 		/*
 		 * Reader does not have this defined.  Must be a single slot
-		 * reader so we can just return SCARD_S_SUCCESS. 
+		 * reader so we can just return SCARD_S_SUCCESS.
 		 */
 		return SCARD_S_SUCCESS;
 
 	if (rv == IFD_SUCCESS && dwGetSize == 1 && ucGetData[0] == 1)
 		/*
-		 * Reader has this defined and it only has one slot 
+		 * Reader has this defined and it only has one slot
 		 */
 		return SCARD_S_SUCCESS;
 
 	/*
-	 * Check the number of slots and create a different 
-	 * structure for each one accordingly 
+	 * Check the number of slots and create a different
+	 * structure for each one accordingly
 	 */
 
 	/*
-	 * Initialize the rest of the slots 
+	 * Initialize the rest of the slots
 	 */
 
 	for (j = 1; j < ucGetData[0]; j++)
@@ -324,8 +326,8 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		DWORD dwContextB = 0;
 
 		/*
-		 * We must find an empty spot to put the 
-		 * reader structure 
+		 * We must find an empty spot to put the
+		 * reader structure
 		 */
 		for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 		{
@@ -339,7 +341,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		if (i == PCSCLITE_MAX_READERS_CONTEXTS)
 		{
 			/*
-			 * No more spots left return 
+			 * No more spots left return
 			 */
 			rv = RFRemoveReader(lpcReader, dwPort);
 			return SCARD_E_NO_MEMORY;
@@ -370,7 +372,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		sReadersContexts[dwContextB]->dwSlot =
 			sReadersContexts[dwContext]->dwSlot + j;
 
-		/* 
+		/*
 		 * Added by Dave - slots did not have a pdwFeeds
 		 * parameter so it was by luck they were working
 		 */
@@ -392,7 +394,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 			(sReadersContexts[dwContextB])->psHandles[i].hCard = 0;
 
 		/*
-		 * Call on the driver to see if the slots are thread safe 
+		 * Call on the driver to see if the slots are thread safe
 		 */
 
 		dwGetSize = sizeof(ucThread);
@@ -417,10 +419,10 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		if (rv != SCARD_S_SUCCESS)
 		{
 			/*
-			 * Cannot connect to slot exit gracefully 
+			 * Cannot connect to slot exit gracefully
 			 */
 			/*
-			 * Clean up so it is not using needed space 
+			 * Clean up so it is not using needed space
 			 */
 			Log2(PCSC_LOG_ERROR, "%s init failed.", lpcReader);
 
@@ -431,7 +433,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 			(sReadersContexts[dwContextB])->dwIdentity = 0;
 
 			/*
-			 * Destroy and free the mutex 
+			 * Destroy and free the mutex
 			 */
 			if (*(sReadersContexts[dwContextB])->pdwMutex == 1)
 			{
@@ -480,7 +482,7 @@ LONG RFRemoveReader(LPSTR lpcReader, DWORD dwPort)
 		int i;
 
 		/*
-		 * Try to destroy the thread 
+		 * Try to destroy the thread
 		 */
 		rv = EHDestroyEventHandler(sContext);
 
@@ -489,7 +491,7 @@ LONG RFRemoveReader(LPSTR lpcReader, DWORD dwPort)
 			return rv;
 
 		/*
-		 * Destroy and free the mutex 
+		 * Destroy and free the mutex
 		 */
 		if ((NULL == sContext->pdwMutex) || (NULL == sContext->pdwFeeds))
 		{
@@ -554,7 +556,7 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 	int i;
 
 	/*
-	 * Clear the list 
+	 * Clear the list
 	 */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 		usedDigits[i] = FALSE;
@@ -571,7 +573,7 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 					LONG ret;
 
 					/*
-					 * Ask the driver if it supports multiple channels 
+					 * Ask the driver if it supports multiple channels
 					 */
 					valueLength = sizeof(tagValue);
 					ret = IFDGetCapabilities((sReadersContexts[i]),
@@ -586,11 +588,11 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 							"Support %d simultaneous readers", tagValue[0]);
 					}
 					else
-						supportedChannels = -1;
+						supportedChannels = 1;
 
 					/*
 					 * Check to see if it is a hotplug reader and
-					 * different 
+					 * different
 					 */
 					if (((((sReadersContexts[i])->dwPort & 0xFFFF0000) ==
 							PCSCLITE_HP_BASE_PORT)
@@ -602,19 +604,19 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 						/*
 						 * tells the caller who the parent of this
 						 * clone is so it can use it's shared
-						 * resources like mutex/etc. 
+						 * resources like mutex/etc.
 						 */
 						parent = i;
 
 						/*
-						 * If the same reader already exists and it is 
+						 * If the same reader already exists and it is
 						 * hotplug then we must look for others and
-						 * enumerate the readername 
+						 * enumerate the readername
 						 */
 						currentDigit = strtol(lpcReader + strlen(lpcReader) - 5, NULL, 16);
 
 						/*
-						 * This spot is taken 
+						 * This spot is taken
 						 */
 						usedDigits[currentDigit] = TRUE;
 					}
@@ -637,14 +639,25 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 				break;
 		}
 
-		if ((i == PCSCLITE_MAX_READERS_CONTEXTS) || (i > supportedChannels))
-			return -1;
+		if (i == PCSCLITE_MAX_READERS_CONTEXTS)
+		{
+			Log2(PCSC_LOG_ERROR, "Max number of readers reached: %d", PCSCLITE_MAX_READERS_CONTEXTS);
+			return -2;
+		}
+
+		if (i >= supportedChannels)
+		{
+			Log3(PCSC_LOG_ERROR, "Driver %s does not support more than "
+				"%d reader(s). Maybe the driver should support "
+				"TAG_IFD_SIMULTANEOUS_ACCESS", libraryName, supportedChannels);
+			return -2;
+		}
 	}
 
 	sprintf(rContext->lpcReader, "%s %02X %02lX", readerName, i, dwSlot);
 
 	/*
-	 * Set the slot in 0xDDDDCCCC 
+	 * Set the slot in 0xDDDDCCCC
 	 */
 	rContext->dwSlot = (i << 16) + dwSlot;
 
@@ -662,7 +675,7 @@ LONG RFListReaders(LPSTR lpcReaders, LPDWORD pdwReaderNum)
 		return SCARD_E_READER_UNAVAILABLE;
 
 	/*
-	 * Ignore the groups for now, return all readers 
+	 * Ignore the groups for now, return all readers
 	 */
 	dwCSize = 0;
 	p = 0;
@@ -678,23 +691,23 @@ LONG RFListReaders(LPSTR lpcReaders, LPDWORD pdwReaderNum)
 
 	if (p > dwNumReadersContexts)
 		/*
-		 * We are severely hosed here 
+		 * We are severely hosed here
 		 */
 		/*
-		 * Hopefully this will never be true 
+		 * Hopefully this will never be true
 		 */
 		return SCARD_F_UNKNOWN_ERROR;
 
 	/*
-	 * Added for extra NULL byte on MultiString 
+	 * Added for extra NULL byte on MultiString
 	 */
 	dwCSize += 1;
 
 	/*
-	 * If lpcReaders is not allocated then just 
+	 * If lpcReaders is not allocated then just
 	 */
 	/*
-	 * return the amount needed to allocate 
+	 * return the amount needed to allocate
 	 */
 	if (lpcReaders == 0)
 	{
@@ -710,7 +723,7 @@ LONG RFListReaders(LPSTR lpcReaders, LPDWORD pdwReaderNum)
 	p = 0;
 
 	/*
-	 * Creating MultiString 
+	 * Creating MultiString
 	 */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
@@ -785,7 +798,7 @@ LONG RFReaderInfoById(DWORD dwIdentity, PREADER_CONTEXT * sReader)
 	int i;
 
 	/*
-	 * Strip off the lower nibble and get the identity 
+	 * Strip off the lower nibble and get the identity
 	 */
 	dwIdentity = dwIdentity >> (sizeof(DWORD) / 2) * 8;
 	dwIdentity = dwIdentity << (sizeof(DWORD) / 2) * 8;
@@ -808,7 +821,7 @@ LONG RFLoadReader(PREADER_CONTEXT rContext)
 	{
 		Log1(PCSC_LOG_ERROR, "Warning library pointer not NULL");
 		/*
-		 * Another reader exists with this library loaded 
+		 * Another reader exists with this library loaded
 		 */
 		return SCARD_S_SUCCESS;
 	}
@@ -824,7 +837,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	/*
 	 * Use this function as a dummy to determine the IFD Handler version
 	 * type  1.0/2.0/3.0.  Suppress error messaging since it can't be 1.0,
-	 * 2.0 and 3.0. 
+	 * 2.0 and 3.0.
 	 */
 
 	DebugLogSuppress(DEBUGLOG_IGNORE_ENTRIES);
@@ -838,7 +851,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	if (rv1 != SCARD_S_SUCCESS && rv2 != SCARD_S_SUCCESS && rv3 != SCARD_S_SUCCESS)
 	{
 		/*
-		 * Neither version of the IFD Handler was found - exit 
+		 * Neither version of the IFD Handler was found - exit
 		 */
 		Log1(PCSC_LOG_CRITICAL, "IFDHandler functions missing");
 
@@ -846,26 +859,26 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	} else if (rv1 == SCARD_S_SUCCESS)
 	{
 		/*
-		 * Ifd Handler 1.0 found 
+		 * Ifd Handler 1.0 found
 		 */
 		rContext->dwVersion = IFD_HVERSION_1_0;
 	} else if (rv3 == SCARD_S_SUCCESS)
 	{
 		/*
-		 * Ifd Handler 3.0 found 
+		 * Ifd Handler 3.0 found
 		 */
 		rContext->dwVersion = IFD_HVERSION_3_0;
 	}
 	else
 	{
 		/*
-		 * Ifd Handler 2.0 found 
+		 * Ifd Handler 2.0 found
 		 */
 		rContext->dwVersion = IFD_HVERSION_2_0;
 	}
 
 	/*
-	 * The following binds version 1.0 of the IFD Handler specs 
+	 * The following binds version 1.0 of the IFD Handler specs
 	 */
 
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
@@ -874,12 +887,12 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 
 #define GET_ADDRESS_OPTIONALv1(field, function, code) \
 { \
-	void *f = NULL; \
-	if (SCARD_S_SUCCESS != DYN_GetAddress(rContext->vHandle, &f, "IFD_" #function)) \
+	void *f1 = NULL; \
+	if (SCARD_S_SUCCESS != DYN_GetAddress(rContext->vHandle, &f1, "IFD_" #function)) \
 	{ \
 		code \
 	} \
-	rContext->psFunctions.psFunctions_v1.pvf ## field = f; \
+	rContext->psFunctions.psFunctions_v1.pvf ## field = f1; \
 }
 
 #define GET_ADDRESSv1(field, function) \
@@ -909,17 +922,17 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	else if (rContext->dwVersion == IFD_HVERSION_2_0)
 	{
 		/*
-		 * The following binds version 2.0 of the IFD Handler specs 
+		 * The following binds version 2.0 of the IFD Handler specs
 		 */
 
 #define GET_ADDRESS_OPTIONALv2(s, code) \
 { \
-	void *f = NULL; \
-	if (SCARD_S_SUCCESS != DYN_GetAddress(rContext->vHandle, &f, "IFDH" #s)) \
+	void *f1 = NULL; \
+	if (SCARD_S_SUCCESS != DYN_GetAddress(rContext->vHandle, &f1, "IFDH" #s)) \
 	{ \
 		code \
 	} \
-	rContext->psFunctions.psFunctions_v2.pvf ## s = f; \
+	rContext->psFunctions.psFunctions_v2.pvf ## s = f1; \
 }
 
 #define GET_ADDRESSv2(s) \
@@ -943,17 +956,17 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	else if (rContext->dwVersion == IFD_HVERSION_3_0)
 	{
 		/*
-		 * The following binds version 3.0 of the IFD Handler specs 
+		 * The following binds version 3.0 of the IFD Handler specs
 		 */
 
 #define GET_ADDRESS_OPTIONALv3(s, code) \
 { \
-	void *f = NULL; \
-	if (SCARD_S_SUCCESS != DYN_GetAddress(rContext->vHandle, &f, "IFDH" #s)) \
+	void *f1 = NULL; \
+	if (SCARD_S_SUCCESS != DYN_GetAddress(rContext->vHandle, &f1, "IFDH" #s)) \
 	{ \
 		code \
 	} \
-	rContext->psFunctions.psFunctions_v3.pvf ## s = f; \
+	rContext->psFunctions.psFunctions_v3.pvf ## s = f1; \
 }
 
 #define GET_ADDRESSv3(s) \
@@ -978,7 +991,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	else
 	{
 		/*
-		 * Who knows what could have happenned for it to get here. 
+		 * Who knows what could have happenned for it to get here.
 		 */
 		Log1(PCSC_LOG_CRITICAL, "IFD Handler not 1.0/2.0 or 3.0");
 		exit(1);
@@ -990,7 +1003,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 LONG RFUnBindFunctions(PREADER_CONTEXT rContext)
 {
 	/*
-	 * Zero out everything 
+	 * Zero out everything
 	 */
 
 	memset(&rContext->psFunctions, 0, sizeof(rContext->psFunctions));
@@ -1001,7 +1014,7 @@ LONG RFUnBindFunctions(PREADER_CONTEXT rContext)
 LONG RFUnloadReader(PREADER_CONTEXT rContext)
 {
 	/*
-	 * Make sure no one else is using this library 
+	 * Make sure no one else is using this library
 	 */
 
 	if (*rContext->pdwFeeds == 1)
@@ -1052,16 +1065,18 @@ LONG RFLockSharing(DWORD hCard)
 LONG RFUnlockSharing(DWORD hCard)
 {
 	PREADER_CONTEXT rContext = NULL;
+	LONG rv;
 
-	RFReaderInfoById(hCard, &rContext);
+	rv = RFReaderInfoById(hCard, &rContext);
+	if (rv != SCARD_S_SUCCESS)
+		return rv;
 
-	if (RFCheckSharing(hCard) == SCARD_S_SUCCESS)
-	{
-		EHSetSharingEvent(rContext, 0);
-		rContext->dwLockId = 0;
-	}
-	else
-		return SCARD_E_SHARING_VIOLATION;
+	rv = RFCheckSharing(hCard);
+	if (rv != SCARD_S_SUCCESS)
+		return rv;
+
+	EHSetSharingEvent(rContext, 0);
+	rContext->dwLockId = 0;
 
 	return SCARD_S_SUCCESS;
 }
@@ -1087,14 +1102,14 @@ LONG RFInitializeReader(PREADER_CONTEXT rContext)
 	LONG rv;
 
 	/*
-	 * Spawn the event handler thread 
+	 * Spawn the event handler thread
 	 */
 	Log3(PCSC_LOG_INFO, "Attempting startup of %s using %s",
 		rContext->lpcReader, rContext->lpcLibrary);
 
   /******************************************/
 	/*
-	 * This section loads the library 
+	 * This section loads the library
 	 */
   /******************************************/
 	rv = RFLoadReader(rContext);
@@ -1103,7 +1118,7 @@ LONG RFInitializeReader(PREADER_CONTEXT rContext)
 
   /*******************************************/
 	/*
-	 * This section binds the functions 
+	 * This section binds the functions
 	 */
   /*******************************************/
 	rv = RFBindFunctions(rContext);
@@ -1116,7 +1131,7 @@ LONG RFInitializeReader(PREADER_CONTEXT rContext)
 
   /*******************************************/
 	/*
-	 * This section tries to open the port 
+	 * This section tries to open the port
 	 */
   /*******************************************/
 
@@ -1140,14 +1155,14 @@ LONG RFUnInitializeReader(PREADER_CONTEXT rContext)
 		rContext->lpcReader);
 
 	/*
-	 * Close the port, unbind the functions, and unload the library 
+	 * Close the port, unbind the functions, and unload the library
 	 */
 
 	/*
 	 * If the reader is getting uninitialized then it is being unplugged
 	 * so I can't send a IFDPowerICC call to it
-	 * 
-	 * IFDPowerICC( rContext, IFD_POWER_DOWN, Atr, &AtrLen ); 
+	 *
+	 * IFDPowerICC( rContext, IFD_POWER_DOWN, Atr, &AtrLen );
 	 */
 	IFDCloseIFD(rContext);
 	RFUnBindFunctions(rContext);
@@ -1162,7 +1177,7 @@ SCARDHANDLE RFCreateReaderHandle(PREADER_CONTEXT rContext)
 
 	/*
 	 * Create a random handle with 16 bits check to see if it already is
-	 * used. 
+	 * used.
 	 */
 	randHandle = SYS_RandomInt(10, 65000);
 
@@ -1182,7 +1197,7 @@ SCARDHANDLE RFCreateReaderHandle(PREADER_CONTEXT rContext)
 						(sReadersContexts[i])->psHandles[j].hCard)
 					{
 						/*
-						 * Get a new handle and loop again 
+						 * Get a new handle and loop again
 						 */
 						randHandle = SYS_RandomInt(10, 65000);
 						continue;
@@ -1193,7 +1208,7 @@ SCARDHANDLE RFCreateReaderHandle(PREADER_CONTEXT rContext)
 
 		/*
 		 * Once the for loop is completed w/o restart a good handle was
-		 * found and the loop can be exited. 
+		 * found and the loop can be exited.
 		 */
 
 		if (i == PCSCLITE_MAX_READERS_CONTEXTS)
@@ -1276,7 +1291,7 @@ LONG RFSetReaderEventState(PREADER_CONTEXT rContext, DWORD dwEvent)
 	int i;
 
 	/*
-	 * Set all the handles for that reader to the event 
+	 * Set all the handles for that reader to the event
 	 */
 	for (i = 0; i < PCSCLITE_MAX_READER_CONTEXT_CHANNELS; i++)
 	{
@@ -1359,7 +1374,7 @@ void RFCleanupReaders(int shouldExit)
 			strncpy(lpcStripReader, (sReadersContexts[i])->lpcReader,
 				sizeof(lpcStripReader));
 			/*
-			 * strip the 6 last char ' 00 00' 
+			 * strip the 6 last char ' 00 00'
 			 */
 			lpcStripReader[strlen(lpcStripReader) - 6] = '\0';
 
@@ -1371,14 +1386,14 @@ void RFCleanupReaders(int shouldExit)
 	}
 
 	/*
-	 * exit() will call at_exit() 
+	 * exit() will call at_exit()
 	 */
 
-	if (shouldExit) 
+	if (shouldExit)
 		exit(0);
 }
 
-int RFStartSerialReaders(char *readerconf)
+int RFStartSerialReaders(const char *readerconf)
 {
 	SerialReader *reader_list;
 	int i, rv;
@@ -1510,7 +1525,7 @@ void RFReCheckReaderConf(void)
 	free(reader_list);
 }
 
-void RFSuspendAllReaders(void) 
+void RFSuspendAllReaders(void)
 {
 	int i;
 
@@ -1525,12 +1540,12 @@ void RFSuspendAllReaders(void)
 
 }
 
-void RFAwakeAllReaders(void) 
+void RFAwakeAllReaders(void)
 {
 	LONG rv = IFD_SUCCESS;
 	int i;
 	int initFlag;
-        
+
 	initFlag = 0;
 
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
@@ -1544,12 +1559,12 @@ void RFAwakeAllReaders(void)
 			for (j=0; j < i; j++)
 			{
 				if (((sReadersContexts[j])->vHandle == (sReadersContexts[i])->vHandle)&&
-					((sReadersContexts[j])->dwPort   == (sReadersContexts[i])->dwPort)) 
+					((sReadersContexts[j])->dwPort   == (sReadersContexts[i])->dwPort))
 				{
 					initFlag = 1;
 				}
 			}
-                        
+
 			if (initFlag == 0)
 				rv = IFDOpenIFD(sReadersContexts[i]);
 			else
