@@ -6,7 +6,7 @@
  * Copyright (C) 1999
  *  David Corcoran <corcoran@linuxnet.com>
  *
- * $Id: sys_unix.c 2076 2006-06-07 12:58:14Z rousseau $
+ * $Id: sys_unix.c 2264 2006-12-03 13:15:03Z rousseau $
  */
 
 /**
@@ -58,7 +58,7 @@ INTERNAL int SYS_Initialize(void)
  * @retval 0 Success.
  * @retval -1 An error occurred.
  */
-INTERNAL int SYS_Mkdir(char *path, int perms)
+INTERNAL int SYS_Mkdir(const char *path, int perms)
 {
 	return mkdir(path, perms);
 }
@@ -121,7 +121,7 @@ INTERNAL int SYS_USleep(int iTimeVal)
  * @retval >0 The file descriptor.
  * @retval -1 An error ocurred.
  */
-INTERNAL int SYS_OpenFile(char *pcFile, int flags, int mode)
+INTERNAL int SYS_OpenFile(const char *pcFile, int flags, int mode)
 {
 	return open(pcFile, flags, mode);
 }
@@ -149,7 +149,7 @@ INTERNAL int SYS_CloseFile(int iHandle)
  * @retval 0 Success.
  * @retval -1 An error ocurred.
  */
-INTERNAL int SYS_RemoveFile(char *pcFile)
+INTERNAL int SYS_RemoveFile(const char *pcFile)
 {
 	return remove(pcFile);
 }
@@ -164,16 +164,6 @@ INTERNAL int SYS_Chdir(const char *path)
 	return chdir(path);
 }
 
-INTERNAL int SYS_Mkfifo(const char *path, int mode)
-{
-	return mkfifo(path, mode);
-}
-
-INTERNAL int SYS_Mknod(const char *path, int mode, int dev)
-{
-	return mknod(path, mode, dev);
-}
-
 INTERNAL int SYS_GetUID(void)
 {
 	return getuid();
@@ -184,91 +174,6 @@ INTERNAL int SYS_GetGID(void)
 	return getgid();
 }
 
-INTERNAL int SYS_Chown(const char *fname, int uid, int gid)
-{
-	return chown(fname, uid, gid);
-}
-
-INTERNAL int SYS_ChangePermissions(char *pcFile, int mode)
-{
-	return chmod(pcFile, mode);
-}
-
-/**
- * @brief Makes a non-blocking request to lock a file exclusively.
- *
- * @param[in] iHandle File descriptor.
- *
- * @return Error code.
- * @retval 0 Success.
- * @retval -1 An error ocurred.
- */
-INTERNAL int SYS_LockFile(int iHandle)
-{
-#ifdef HAVE_FLOCK
-	return flock(iHandle, LOCK_EX | LOCK_NB);
-#else
-	struct flock lock_s;
-
-	lock_s.l_type = F_WRLCK;
-	lock_s.l_whence = 0;
-	lock_s.l_start = 0L;
-	lock_s.l_len = 0L;
-
-	return fcntl(iHandle, F_SETLK, &lock_s);
-#endif
-}
-
-/**
- * @brief Makes a blocking request to lock a file exclusively.
- *
- * @param[in] iHandle File descriptor.
- *
- * @return Error code.
- * @retval 0 Success.
- * @retval -1 An error ocurred.
- */
-INTERNAL int SYS_LockAndBlock(int iHandle)
-{
-#ifdef HAVE_FLOCK
-	return flock(iHandle, LOCK_EX);
-#else
-	struct flock lock_s;
-
-	lock_s.l_type = F_RDLCK;
-	lock_s.l_whence = 0;
-	lock_s.l_start = 0L;
-	lock_s.l_len = 0L;
-
-	return fcntl(iHandle, F_SETLKW, &lock_s);
-#endif
-}
-
-/**
- * @brief Unlocks the file.
- *
- * @param[in] iHandle File descriptor.
- *
- * @return Error code.
- * @retval 0 Success.
- * @retval -1 An error ocurred.
- */
-INTERNAL int SYS_UnlockFile(int iHandle)
-{
-#ifdef HAVE_FLOCK
-	return flock(iHandle, LOCK_UN);
-#else
-	struct flock lock_s;
-
-	lock_s.l_type = F_UNLCK;
-	lock_s.l_whence = 0;
-	lock_s.l_start = 0L;
-	lock_s.l_len = 0L;
-
-	return fcntl(iHandle, F_SETLK, &lock_s);
-#endif
-}
-
 INTERNAL int SYS_SeekFile(int iHandle, int iSeekLength)
 {
 	int iOffset;
@@ -281,7 +186,7 @@ INTERNAL int SYS_ReadFile(int iHandle, char *pcBuffer, int iLength)
 	return read(iHandle, pcBuffer, iLength);
 }
 
-INTERNAL int SYS_WriteFile(int iHandle, char *pcBuffer, int iLength)
+INTERNAL int SYS_WriteFile(int iHandle, const char *pcBuffer, int iLength)
 {
 	return write(iHandle, pcBuffer, iLength);
 }
@@ -289,7 +194,7 @@ INTERNAL int SYS_WriteFile(int iHandle, char *pcBuffer, int iLength)
 /**
  * @brief Gets the memory page size.
  *
- * The page size is used when calling the \c SYS_MemoryMap() and 
+ * The page size is used when calling the \c SYS_MemoryMap() and
  * \c SYS_PublicMemoryMap() functions.
  *
  * @return Number of bytes per page.
@@ -321,8 +226,8 @@ INTERNAL void *SYS_MemoryMap(int iSize, int iFid, int iOffset)
 	/*
 	 * Here are some common error types: switch( errno ) { case EINVAL:
 	 * printf("EINVAL"); case EBADF: printf("EBADF"); break; case EACCES:
-	 * printf("EACCES"); break; case EAGAIN: printf("EAGAIN"); break; case 
-	 * ENOMEM: printf("ENOMEM"); break; } 
+	 * printf("EACCES"); break; case EAGAIN: printf("EAGAIN"); break; case
+	 * ENOMEM: printf("ENOMEM"); break; }
 	 */
 
 	return vAddress;
@@ -344,7 +249,7 @@ INTERNAL void *SYS_PublicMemoryMap(int iSize, int iFid, int iOffset)
 
 	vAddress = 0;
 	vAddress = mmap(0, iSize, PROT_READ, MAP_SHARED, iFid, iOffset);
-	if (vAddress == (void*)-1) // mmap returns -1 on error
+	if (vAddress == (void*)-1) /* mmap returns -1 on error */
 	{
 		Log2(PCSC_LOG_CRITICAL, "SYS_PublicMemoryMap() failed: %s",
 			strerror(errno));
@@ -421,7 +326,7 @@ INTERNAL int SYS_Daemon(int nochdir, int noclose)
 			/* the father exits */
 			exit(0);
 	}
-	
+
 	setsid();
 
 	pid = SYS_Fork();
@@ -469,22 +374,11 @@ INTERNAL int SYS_Daemon(int nochdir, int noclose)
 #endif
 }
 
-INTERNAL int SYS_Wait(int iPid, int iWait)
-{
-	return waitpid(-1, 0, WNOHANG);
-}
-
-INTERNAL int SYS_Stat(char *pcFile, struct stat *psStatus)
+INTERNAL int SYS_Stat(const char *pcFile, struct stat *psStatus)
 {
 	return stat(pcFile, psStatus);
 }
 
-INTERNAL int SYS_Fstat(int iFd)
-{
-	struct stat sStatus;
-	return fstat(iFd, &sStatus);
-}
-
 INTERNAL int SYS_RandomInt(int fStart, int fEnd)
 {
 	static int iInitialized = 0;
@@ -524,12 +418,7 @@ INTERNAL void SYS_Exit(int iRetVal)
 	_exit(iRetVal);
 }
 
-INTERNAL int SYS_Rmdir(char *pcFile)
-{
-	return rmdir(pcFile);
-}
-
-INTERNAL int SYS_Unlink(char *pcFile)
+INTERNAL int SYS_Unlink(const char *pcFile)
 {
 	return unlink(pcFile);
 }
