@@ -3,10 +3,10 @@
  *
  * Copyright (C) 1999
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2004
+ * Copyright (C) 2004-2006
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: testpcsc.c 2114 2006-07-31 13:28:50Z rousseau $
+ * $Id: testpcsc.c 2320 2007-01-09 15:28:13Z rousseau $
  */
 
 /**
@@ -23,19 +23,41 @@
 #include "winscard.h"
 #include "reader.h"
 
+#define PANIC 0
+#define DONT_PANIC 1
+#define RED_PRINTF_FORMAT "\33[01;31m%s\33[0m\n"
+
+void test_rv(int rv, SCARDCONTEXT hContext, int dont_panic)
+{
+	if (rv != SCARD_S_SUCCESS)
+	{
+		if (dont_panic)
+			printf("\33[34m%s (don't panic)\33[0m\n", pcsc_stringify_error(rv));
+		else
+		{
+			printf(RED_PRINTF_FORMAT, pcsc_stringify_error(rv));
+			SCardReleaseContext(hContext);
+			exit(-1);
+		}
+	}
+	else
+		puts(pcsc_stringify_error(rv));
+}
+
 int main(int argc, char **argv)
 {
 	SCARDHANDLE hCard;
 	SCARDCONTEXT hContext;
 	SCARD_READERSTATE_A rgReaderStates[1];
-	unsigned long dwReaderLen, dwState, dwProt, dwAtrLen;
-	unsigned long dwPref, dwReaders;
+	DWORD dwReaderLen, dwState, dwProt, dwAtrLen;
+	DWORD dwPref, dwReaders;
 	char *pcReaders, *mszReaders;
 	unsigned char pbAtr[MAX_ATR_SIZE];
 	char *mszGroups;
-	unsigned long dwGroups;
+	DWORD dwGroups;
 	long rv;
-	int i, p, iReader;
+	DWORD i;
+	int p, iReader;
 	int iList[16];
 	SCARD_IO_REQUEST pioRecvPci;
 	SCARD_IO_REQUEST pioSendPci;
@@ -45,55 +67,37 @@ int main(int argc, char **argv)
 
 	printf("\nMUSCLE PC/SC Lite unitary test Program\n\n");
 
-	printf("THIS PROGRAM IS NOT DESIGNED AS A TESTING TOOL FOR END USERS!\n");
-	printf("Do NOT use it unless you really know what you do.\n\n");
+	printf("\33[35mTHIS PROGRAM IS NOT DESIGNED AS A TESTING TOOL FOR END USERS!\n");
+	printf("Do NOT use it unless you really know what you do.\33[0m\n\n");
 
-	printf("Testing SCardEstablishContext    : ");
+	printf("Testing SCardEstablishContext\t: ");
 	rv = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &hContext);
+	test_rv(rv, hContext, PANIC);
 
-	printf("%s\n", pcsc_stringify_error(rv));
+	printf("Testing SCardIsValidContext\t: ");
+	rv = SCardIsValidContext(hContext);
+	test_rv(rv, hContext, PANIC);
 
-	if (rv != SCARD_S_SUCCESS)
-	{
-		return -1;
-	}
+	printf("Testing SCardIsValidContext\t: ");
+	rv = SCardIsValidContext(hContext+1);
+	test_rv(rv, hContext, DONT_PANIC);
 
 	printf("Testing SCardGetStatusChange \n");
-	printf("Please insert a working reader   : ");
+	printf("Please insert a working reader\t: ");
 	fflush(stdout);
 	rv = SCardGetStatusChange(hContext, INFINITE, 0, 0);
+	test_rv(rv, hContext, PANIC);
 
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
-
-	printf("Testing SCardListReaderGroups    : ");
-
+	printf("Testing SCardListReaderGroups\t: ");
 	rv = SCardListReaderGroups(hContext, 0, &dwGroups);
-
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
+	test_rv(rv, hContext, PANIC);
 
 	mszGroups = malloc(sizeof(char) * dwGroups);
 	rv = SCardListReaderGroups(hContext, mszGroups, &dwGroups);
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
+	test_rv(rv, hContext, PANIC);
 
 	/*
-	 * Have to understand the multi-string here 
+	 * Have to understand the multi-string here
 	 */
 	p = 0;
 	for (i = 0; i < dwGroups - 1; i++)
@@ -104,30 +108,18 @@ int main(int argc, char **argv)
 		while (mszGroups[++i] != 0) ;
 	}
 
-	printf("Testing SCardListReaders         : ");
+	printf("Testing SCardListReaders\t: ");
 
 	mszGroups = 0;
 	rv = SCardListReaders(hContext, mszGroups, 0, &dwReaders);
-
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
+	test_rv(rv, hContext, PANIC);
 
 	mszReaders = malloc(sizeof(char) * dwReaders);
 	rv = SCardListReaders(hContext, mszGroups, mszReaders, &dwReaders);
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
+	test_rv(rv, hContext, PANIC);
 
 	/*
-	 * Have to understand the multi-string here 
+	 * Have to understand the multi-string here
 	 */
 	p = 0;
 	for (i = 0; i < dwReaders - 1; i++)
@@ -141,9 +133,11 @@ int main(int argc, char **argv)
 	if (p > 1)
 		do
 		{
-			printf("Enter the reader number          : ");
-			scanf("%d", &iReader);
-			printf("\n");
+			char input[80];
+
+			printf("Enter the reader number\t\t: ");
+			fgets(input, sizeof(input), stdin);
+			sscanf(input, "%d", &iReader);
 
 			if (iReader > p || iReader <= 0)
 				printf("Invalid Value - try again\n");
@@ -155,30 +149,16 @@ int main(int argc, char **argv)
 	rgReaderStates[0].szReader = &mszReaders[iList[iReader]];
 	rgReaderStates[0].dwCurrentState = SCARD_STATE_EMPTY;
 
-	printf("Waiting for card insertion       : ");
+	printf("Waiting for card insertion\t: ");
 	fflush(stdout);
 	rv = SCardGetStatusChange(hContext, INFINITE, rgReaderStates, 1);
+	test_rv(rv, hContext, PANIC);
 
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
-
-	printf("Testing SCardConnect             : ");
+	printf("Testing SCardConnect\t\t: ");
 	rv = SCardConnect(hContext, &mszReaders[iList[iReader]],
 		SCARD_SHARE_SHARED, SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1,
 		&hCard, &dwPref);
-
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
+	test_rv(rv, hContext, PANIC);
 
 	switch(dwPref)
 	{
@@ -202,7 +182,7 @@ int main(int argc, char **argv)
 	printf("\n");
 	length = sizeof(bRecvBuffer);
 
-	printf("Testing SCardTransmit            : ");
+	printf("Testing SCardTransmit\t\t: ");
 	rv = SCardTransmit(hCard, &pioSendPci, bSendBuffer, send_length,
 		&pioRecvPci, bRecvBuffer, &length);
 	printf("%s\n", pcsc_stringify_error(rv));
@@ -211,7 +191,7 @@ int main(int argc, char **argv)
 		printf(" %02X", bRecvBuffer[i]);
 	printf("\n");
 
-	printf("Testing SCardControl             : ");
+	printf("Testing SCardControl\t\t: ");
 #ifdef PCSC_PRE_120
 	{
 		char buffer[1024] = "Foobar";
@@ -234,18 +214,18 @@ int main(int argc, char **argv)
 		}
 	}
 #endif
-	printf("%s %s\n", pcsc_stringify_error(rv), rv != SCARD_S_SUCCESS ? "(don't panic)" : "");
+	test_rv(rv, hContext, DONT_PANIC);
 
-	printf("Testing SCardGetAttrib           : ");
+	printf("Testing SCardGetAttrib\t\t: ");
 	rv = SCardGetAttrib(hCard, SCARD_ATTR_ATR_STRING, NULL, &dwAtrLen);
-	printf("%s %s\n", pcsc_stringify_error(rv), rv != SCARD_S_SUCCESS ? "(don't panic)" : "");
+	test_rv(rv, hContext, DONT_PANIC);
 	if (rv == SCARD_S_SUCCESS)
 		printf("ATR length: %ld\n", dwAtrLen);
 
-	printf("Testing SCardGetAttrib           : ");
+	printf("Testing SCardGetAttrib\t\t: ");
 	dwAtrLen = sizeof(pbAtr);
 	rv = SCardGetAttrib(hCard, SCARD_ATTR_ATR_STRING, pbAtr, &dwAtrLen);
-	printf("%s %s\n", pcsc_stringify_error(rv), rv != SCARD_S_SUCCESS ? "(don't panic)" : "");
+	test_rv(rv, hContext, DONT_PANIC);
 	if (rv == SCARD_S_SUCCESS)
 	{
 		for (i = 0; i < dwAtrLen; i++)
@@ -253,11 +233,37 @@ int main(int argc, char **argv)
 		printf("\n");
 	}
 
-	printf("Testing SCardSetAttrib           : ");
-	rv = SCardSetAttrib(hCard, SCARD_ATTR_ATR_STRING, (LPCBYTE)"", 1);
-	printf("%s %s\n", pcsc_stringify_error(rv), rv != SCARD_S_SUCCESS ? "(don't panic)" : "");
+	printf("Testing SCardGetAttrib\t\t: ");
+	dwAtrLen = sizeof(pbAtr);
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_VENDOR_IFD_VERSION, pbAtr, &dwAtrLen);
+	test_rv(rv, hContext, DONT_PANIC);
+	if (rv == SCARD_S_SUCCESS)
+		printf("Vendor IFD version\t\t: 0x%08lX\n", ((DWORD *)pbAtr)[0]);
 
-	printf("Testing SCardStatus              : ");
+	printf("Testing SCardGetAttrib\t\t: ");
+	dwAtrLen = sizeof(pbAtr);
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_MAXINPUT, pbAtr, &dwAtrLen);
+	test_rv(rv, hContext, DONT_PANIC);
+	if (rv == SCARD_S_SUCCESS)
+	{
+		if (dwAtrLen == sizeof(uint32_t))
+			printf("Max message length\t\t: %d\n", *(uint32_t *)pbAtr);
+		else
+			printf(RED_PRINTF_FORMAT, "Wrong size");
+	}
+
+	printf("Testing SCardGetAttrib\t\t: ");
+	dwAtrLen = sizeof(pbAtr);
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_VENDOR_NAME, pbAtr, &dwAtrLen);
+	test_rv(rv, hContext, DONT_PANIC);
+	if (rv == SCARD_S_SUCCESS)
+		printf("Vendor name\t\t\t: %s\n", pbAtr);
+
+	printf("Testing SCardSetAttrib\t\t: ");
+	rv = SCardSetAttrib(hCard, SCARD_ATTR_ATR_STRING, (LPCBYTE)"", 1);
+	test_rv(rv, hContext, DONT_PANIC);
+
+	printf("Testing SCardStatus\t\t: ");
 
 	dwReaderLen = 50;
 	pcReaders   = malloc(sizeof(char) * 50);
@@ -265,14 +271,13 @@ int main(int argc, char **argv)
 
 	rv = SCardStatus(hCard, pcReaders, &dwReaderLen, &dwState, &dwProt,
 		pbAtr, &dwAtrLen);
+	test_rv(rv, hContext, PANIC);
 
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	printf("Current Reader Name              : %s\n", pcReaders);
-	printf("Current Reader State             : 0x%.4lx\n", dwState);
-	printf("Current Reader Protocol          : T=%ld\n", dwProt - 1);
-	printf("Current Reader ATR Size          : %ld bytes\n", dwAtrLen);
-	printf("Current Reader ATR Value         : ");
+	printf("Current Reader Name\t\t: %s\n", pcReaders);
+	printf("Current Reader State\t\t: 0x%.4lx\n", dwState);
+	printf("Current Reader Protocol\t\t: T=%ld\n", dwProt - 1);
+	printf("Current Reader ATR Size\t\t: %ld bytes\n", dwAtrLen);
+	printf("Current Reader ATR Value\t: ");
 
 	for (i = 0; i < dwAtrLen; i++)
 	{
@@ -288,38 +293,18 @@ int main(int argc, char **argv)
 
 	printf("Press enter: ");
 	getchar();
-	printf("Testing SCardReconnect           : ");
+	printf("Testing SCardReconnect\t\t: ");
 	rv = SCardReconnect(hCard, SCARD_SHARE_SHARED,
 		SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1, SCARD_UNPOWER_CARD, &dwPref);
+	test_rv(rv, hContext, PANIC);
 
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
-
-	printf("Testing SCardDisconnect          : ");
+	printf("Testing SCardDisconnect\t\t: ");
 	rv = SCardDisconnect(hCard, SCARD_UNPOWER_CARD);
+	test_rv(rv, hContext, PANIC);
 
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		SCardReleaseContext(hContext);
-		return -1;
-	}
-
-	printf("Testing SCardReleaseContext      : ");
+	printf("Testing SCardReleaseContext\t: ");
 	rv = SCardReleaseContext(hContext);
-
-	printf("%s\n", pcsc_stringify_error(rv));
-
-	if (rv != SCARD_S_SUCCESS)
-	{
-		return -1;
-	}
+	test_rv(rv, hContext, PANIC);
 
 	printf("\n");
 	printf("PC/SC Test Completed Successfully !\n");
