@@ -5,12 +5,12 @@
  *  David Corcoran <corcoran@linuxnet.com>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard.c 2072 2006-06-06 09:31:07Z rousseau $
+ * $Id: winscard.c 2274 2006-12-12 13:08:22Z rousseau $
  */
 
 /**
  * @mainpage MUSCLE PC/SC-Lite API Documentation
- * 
+ *
  * @section Introduction
  *
  * This document contains the reference API calls for communicating to the
@@ -29,7 +29,7 @@
  *
  * I would really like to hear from you. If you have any feedback either on
  * this documentation or on the MUSCLE project please feel free to email me at:
- * corcoran@musclecard.com. 
+ * corcoran@musclecard.com.
  *
  *
  * @section API Routines
@@ -45,10 +45,10 @@
  *
  * For a human readable representation of an error the function
  * pcsc_stringify_error() is declared in pcsclite.h. This function is not
- * available on Microsoft(R) winscard API and is pcsc-lite specific. 
+ * available on Microsoft(R) winscard API and is pcsc-lite specific.
  *
  * @section Internals
- * 
+ *
  * PC/SC Lite is formed by a server deamon (<tt>pcscd</tt>) and a client
  * library (<tt>libpcsclite.so</tt>) that communicate via IPC.
  *
@@ -115,7 +115,7 @@ SCARD_IO_REQUEST g_rgSCardRawPci = { SCARD_PROTOCOL_RAW, 8 };
  *
  * This must be the first function called in a PC/SC application.
  *
- * @param[in] dwScope Scope of the establishment. 
+ * @param[in] dwScope Scope of the establishment.
  * This can either be a local or remote connection.
  * <ul>
  *   <li>SCARD_SCOPE_USER - Not used.
@@ -128,9 +128,9 @@ SCARD_IO_REQUEST g_rgSCardRawPci = { SCARD_PROTOCOL_RAW, 8 };
  * @param[out] phContext Returned Application Context.
  *
  * @return Connection status.
- * @retval SCARD_S_SUCCESS Successful.
- * @retval SCARD_E_INVALID_VALUE Invalid scope type passed.
- * @retval SCARD_E_INVALID_PARAMETER phContext is null.
+ * @retval SCARD_S_SUCCESS Successful (\ref SCARD_S_SUCCESS)
+ * @retval SCARD_E_INVALID_VALUE Invalid scope type passed (\ref SCARD_E_INVALID_VALUE)
+ * @retval SCARD_E_INVALID_PARAMETER phContext is null (\ref SCARD_E_INVALID_PARAMETER)
  */
 LONG SCardEstablishContext(DWORD dwScope, LPCVOID pvReserved1,
 	LPCVOID pvReserved2, LPSCARDCONTEXT phContext)
@@ -242,6 +242,20 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 		return SCARD_E_SHARING_VIOLATION;
 	}
 
+	/*
+	 * wait until a possible transaction is finished
+	 */
+	if (rContext->dwLockId != 0)
+	{
+		Log1(PCSC_LOG_INFO, "Waiting for release of lock");
+		while (rContext->dwLockId != 0)
+			SYS_USleep(100000);
+		Log1(PCSC_LOG_INFO, "Lock released");
+
+		/* Allow the status thread to convey information */
+		SYS_USleep(PCSCLITE_STATUS_POLL_RATE + 10);
+	}
+
 	/*******************************************
 	 *
 	 * This section tries to determine the
@@ -442,7 +456,8 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 	/*
 	 * Make sure no one has a lock on this reader
 	 */
-	if ((rv = RFCheckSharing(hCard)) != SCARD_S_SUCCESS)
+	rv = RFCheckSharing(hCard);
+	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
 	/*
@@ -702,10 +717,23 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 		&& (dwDisposition != SCARD_EJECT_CARD))
 		return SCARD_E_INVALID_VALUE;
 
+	/*
+	 * wait until a possible transaction is finished
+	 */
+	if ((rContext->dwLockId != 0) && (rContext->dwLockId != hCard))
+	{
+		Log1(PCSC_LOG_INFO, "Waiting for release of lock");
+		while (rContext->dwLockId != 0)
+			SYS_USleep(100000);
+		Log1(PCSC_LOG_INFO, "Lock released");
+	}
+
 	/*
 	 * Unlock any blocks on this context
 	 */
-	RFUnlockSharing(hCard);
+	rv = RFUnlockSharing(hCard);
+	if (rv != SCARD_S_SUCCESS)
+		return rv;
 
 	Log2(PCSC_LOG_DEBUG, "Active Contexts: %d", rContext->dwContexts);
 
@@ -1083,8 +1111,7 @@ LONG SCardStatus(SCARDHANDLE hCard, LPSTR mszReaderNames,
 		return rv;
 
 	if (strlen(rContext->lpcReader) > MAX_BUFFER_SIZE
-			|| rContext->readerState->cardAtrLength > MAX_ATR_SIZE
-			|| rContext->readerState->cardAtrLength < 0)
+			|| rContext->readerState->cardAtrLength > MAX_ATR_SIZE)
 		return SCARD_F_INTERNAL_ERROR;
 
 	/*
@@ -1233,9 +1260,6 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode,
 	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
 		return rv;
 
-	if (cbSendLength > MAX_BUFFER_SIZE)
-		return SCARD_E_INSUFFICIENT_BUFFER;
-
 	if (IFD_HVERSION_2_0 == rContext->dwVersion)
 	{
 		/* we must wrap a API 3.0 client in an API 2.0 driver */
@@ -1291,7 +1315,10 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	if (rv == IFD_SUCCESS)
 		return SCARD_S_SUCCESS;
 	else
-		return SCARD_E_NOT_TRANSACTED;
+		if (rv == IFD_ERROR_TAG)
+			return SCARD_E_UNSUPPORTED_FEATURE;
+		else
+			return SCARD_E_NOT_TRANSACTED;
 }
 
 LONG SCardSetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
@@ -1334,7 +1361,10 @@ LONG SCardSetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	if (rv == IFD_SUCCESS)
 		return SCARD_S_SUCCESS;
 	else
-		return SCARD_E_NOT_TRANSACTED;
+		if (rv == IFD_ERROR_TAG)
+			return SCARD_E_UNSUPPORTED_FEATURE;
+		else
+			return SCARD_E_NOT_TRANSACTED;
 }
 
 LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
