@@ -6,14 +6,14 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludoic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg.c 2074 2006-06-06 09:42:01Z rousseau $
+ * $Id: winscard_msg.c 2321 2007-01-09 15:30:45Z rousseau $
  */
 
 /**
  * @file
  * @brief This is responsible for client/server communication.
  *
- * A file based socket (\c commonSocket) is used to send/receive only messages 
+ * A file based socket (\c commonSocket) is used to send/receive only messages
  * among clients and server.\n
  * The messages' data are passed throw a memory mapped file: \c sharedSegmentMsg.
  */
@@ -65,7 +65,7 @@ INTERNAL int SHMClientRead(psharedSegmentMsg msgStruct, DWORD dwClientID, int bl
  * server. The socket is associated to the file \c PCSCLITE_CSOCK_NAME.
  *
  * @param[out] pdwClientID Client Connection ID.
- * 
+ *
  * @retval 0 Success.
  * @retval -1 Can not create the socket.
  * @retval -1 The socket can not open a connection.
@@ -75,13 +75,16 @@ INTERNAL int SHMClientSetupSession(PDWORD pdwClientID)
 {
 	struct sockaddr_un svc_addr;
 	int one;
+	int ret;
 
-	if ((*pdwClientID = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
+	ret = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (ret < 0)
 	{
 		Log2(PCSC_LOG_CRITICAL, "Error: create on client socket: %s",
 			strerror(errno));
 		return -1;
 	}
+	*pdwClientID = ret;
 
 	svc_addr.sun_family = AF_UNIX;
 	strncpy(svc_addr.sun_path, PCSCLITE_CSOCK_NAME,
@@ -126,7 +129,7 @@ INTERNAL int SHMClientCloseSession(DWORD dwClientID)
  *
  * Writes the message in the shared file \c filedes.
  *
- * @param[in] buffer Message to be sent.
+ * @param[in] buffer_void Message to be sent.
  * @param[in] buffer_size Size of the message to send
  * @param[in] filedes Socket handle.
  * @param[in] blockAmount Timeout in milliseconds.
@@ -136,24 +139,26 @@ INTERNAL int SHMClientCloseSession(DWORD dwClientID)
  * @retval -1 Socket is closed.
  * @retval -1 A signal was received.
  */
-INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
+INTERNAL int SHMMessageSend(void *buffer_void, size_t buffer_size,
 	int filedes, int blockAmount)
 {
+	char *buffer = buffer_void;
+
 	/*
-	 * default is success 
+	 * default is success
 	 */
 	int retval = 0;
 	/*
-	 * record the time when we started 
+	 * record the time when we started
 	 */
 	time_t start = time(0);
 	/*
-	 * how many bytes remains to be written 
+	 * how many bytes remains to be written
 	 */
 	size_t remaining = buffer_size;
 
 	/*
-	 * repeat until all data is written 
+	 * repeat until all data is written
 	 */
 	while (remaining > 0)
 	{
@@ -168,7 +173,7 @@ INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
 		if ((timeout.tv_sec = start + blockAmount - time(0)) < 0)
 		{
 			/*
-			 * we already timed out 
+			 * we already timed out
 			 */
 			retval = -1;
 			break;
@@ -177,7 +182,7 @@ INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
 		selret = select(filedes + 1, NULL, &write_fd, NULL, &timeout);
 
 		/*
-		 * try to write only when the file descriptor is writable 
+		 * try to write only when the file descriptor is writable
 		 */
 		if (selret > 0)
 		{
@@ -186,7 +191,7 @@ INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
 			if (!FD_ISSET(filedes, &write_fd))
 			{
 				/*
-				 * very strange situation. it should be an assert really 
+				 * very strange situation. it should be an assert really
 				 */
 				retval = -1;
 				break;
@@ -196,14 +201,14 @@ INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
 			if (written > 0)
 			{
 				/*
-				 * we wrote something 
+				 * we wrote something
 				 */
 				buffer += written;
 				remaining -= written;
 			} else if (written == 0)
 			{
 				/*
-				 * peer closed the socket 
+				 * peer closed the socket
 				 */
 				retval = -1;
 				break;
@@ -211,7 +216,7 @@ INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
 			{
 				/*
 				 * we ignore the signals and socket full situations, all
-				 * other errors are fatal 
+				 * other errors are fatal
 				 */
 				if (errno != EINTR && errno != EAGAIN)
 				{
@@ -222,14 +227,14 @@ INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
 		} else if (selret == 0)
 		{
 			/*
-			 * timeout 
+			 * timeout
 			 */
 			retval = -1;
 			break;
 		} else
 		{
 			/*
-			 * ignore signals 
+			 * ignore signals
 			 */
 			if (errno != EINTR)
 			{
@@ -249,7 +254,7 @@ INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
  *
  * Reads the message from the file \c filedes.
  *
- * @param[out] buffer Message read.
+ * @param[out] buffer_void Message read.
  * @param[in] buffer_size Size to read
  * @param[in] filedes Socket handle.
  * @param[in] blockAmount Timeout in milliseconds.
@@ -259,24 +264,26 @@ INTERNAL int SHMMessageSend(void *buffer, size_t buffer_size,
  * @retval -1 Socket is closed.
  * @retval -1 A signal was received.
  */
-INTERNAL int SHMMessageReceive(void *buffer, size_t buffer_size,
+INTERNAL int SHMMessageReceive(void *buffer_void, size_t buffer_size,
 	int filedes, int blockAmount)
 {
+	char *buffer = buffer_void;
+
 	/*
-	 * default is success 
+	 * default is success
 	 */
 	int retval = 0;
 	/*
-	 * record the time when we started 
+	 * record the time when we started
 	 */
 	time_t start = time(0);
 	/*
-	 * how many bytes we must read 
+	 * how many bytes we must read
 	 */
 	size_t remaining = buffer_size;
 
 	/*
-	 * repeate until we get the whole message 
+	 * repeate until we get the whole message
 	 */
 	while (remaining > 0)
 	{
@@ -291,7 +298,7 @@ INTERNAL int SHMMessageReceive(void *buffer, size_t buffer_size,
 		if ((timeout.tv_sec = start + blockAmount - time(0)) < 0)
 		{
 			/*
-			 * we already timed out 
+			 * we already timed out
 			 */
 			retval = -1;
 			break;
@@ -300,7 +307,7 @@ INTERNAL int SHMMessageReceive(void *buffer, size_t buffer_size,
 		selret = select(filedes + 1, &read_fd, NULL, NULL, &timeout);
 
 		/*
-		 * try to read only when socket is readable 
+		 * try to read only when socket is readable
 		 */
 		if (selret > 0)
 		{
@@ -309,7 +316,7 @@ INTERNAL int SHMMessageReceive(void *buffer, size_t buffer_size,
 			if (!FD_ISSET(filedes, &read_fd))
 			{
 				/*
-				 * very strange situation. it should be an assert really 
+				 * very strange situation. it should be an assert really
 				 */
 				retval = -1;
 				break;
@@ -319,14 +326,14 @@ INTERNAL int SHMMessageReceive(void *buffer, size_t buffer_size,
 			if (readed > 0)
 			{
 				/*
-				 * we got something 
+				 * we got something
 				 */
 				buffer += readed;
 				remaining -= readed;
 			} else if (readed == 0)
 			{
 				/*
-				 * peer closed the socket 
+				 * peer closed the socket
 				 */
 				retval = -1;
 				break;
@@ -334,7 +341,7 @@ INTERNAL int SHMMessageReceive(void *buffer, size_t buffer_size,
 			{
 				/*
 				 * we ignore the signals and empty socket situations, all
-				 * other errors are fatal 
+				 * other errors are fatal
 				 */
 				if (errno != EINTR && errno != EAGAIN)
 				{
@@ -345,14 +352,14 @@ INTERNAL int SHMMessageReceive(void *buffer, size_t buffer_size,
 		} else if (selret == 0)
 		{
 			/*
-			 * timeout 
+			 * timeout
 			 */
 			retval = -1;
 			break;
 		} else
 		{
 			/*
-			 * we ignore signals, all other errors are fatal 
+			 * we ignore signals, all other errors are fatal
 			 */
 			if (errno != EINTR)
 			{
@@ -378,18 +385,20 @@ INTERNAL int SHMMessageReceive(void *buffer, size_t buffer_size,
  * @param[in] dwClientID Client socket handle.
  * @param[in] size Size of the message (\p data).
  * @param[in] blockAmount Timeout to the operation in ms.
- * @param[in] data Data to be sent.
+ * @param[in] data_void Data to be sent.
  *
  * @return Same error codes as SHMMessageSend().
  */
 INTERNAL int WrapSHMWrite(unsigned int command, DWORD dwClientID,
-	unsigned int size, unsigned int blockAmount, void *data)
+	unsigned int size, unsigned int blockAmount, void *data_void)
 {
+	char *data = data_void;
+
 	sharedSegmentMsg msgStruct;
 	int ret;
 
 	/*
-	 * Set the appropriate packet parameters 
+	 * Set the appropriate packet parameters
 	 */
 
 	memset(&msgStruct, 0, sizeof(msgStruct));
@@ -398,7 +407,8 @@ INTERNAL int WrapSHMWrite(unsigned int command, DWORD dwClientID,
 	msgStruct.group_id = SYS_GetGID();
 	msgStruct.command = command;
 	msgStruct.date = time(NULL);
-	if (SCARD_TRANSMIT_EXTENDED == command)
+	if ((SCARD_TRANSMIT_EXTENDED == command)
+		|| (SCARD_CONTROL_EXTENDED == command))
 	{
 		/* first block */
 		memcpy(msgStruct.data, data, PCSCLITE_MAX_MESSAGE_SIZE);
@@ -436,7 +446,7 @@ INTERNAL int WrapSHMWrite(unsigned int command, DWORD dwClientID,
  * @param[in] sockValue Socket to be closed.
  * @param[in] pcFilePath File used by the socket.
  */
-INTERNAL void SHMCleanupSharedSegment(int sockValue, char *pcFilePath)
+INTERNAL void SHMCleanupSharedSegment(int sockValue, const char *pcFilePath)
 {
 	SYS_CloseFile(sockValue);
 	SYS_Unlink(pcFilePath);
