@@ -6,14 +6,14 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludoic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg_srv.c,v 1.1.1.4 2006-08-11 18:49:09 rousseau Exp $
+ * $Id: winscard_msg_srv.c,v 1.6 2007-01-25 14:55:37 rousseau Exp $
  */
 
 /**
  * @file
  * @brief client/server communication (on the server side only)
  *
- * A file based socket (\c commonSocket) is used to send/receive only messages 
+ * A file based socket (\c commonSocket) is used to send/receive only messages
  * among clients and server.\n
  * The messages' data are passed throw a memory mapped file: \c sharedSegmentMsg.
  */
@@ -100,7 +100,7 @@ static int SHMProcessCommonChannelRequest(PDWORD pdwClientID)
  * This is called by the server to create a socket for local IPC with the
  * clients. The socket is associated to the file \c PCSCLITE_CSOCK_NAME.
  * Each client will open a connection to this socket.
- * 
+ *
  * @return Error code.
  * @retval 0 Success
  * @retval -1 Can not create the socket.
@@ -112,7 +112,7 @@ INTERNAL int SHMInitializeCommonSegment(void)
 	static struct sockaddr_un serv_adr;
 
 	/*
-	 * Create the common shared connection socket 
+	 * Create the common shared connection socket
 	 */
 	if ((commonSocket = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
 	{
@@ -144,7 +144,7 @@ INTERNAL int SHMInitializeCommonSegment(void)
 	}
 
 	/*
-	 * Chmod the public entry channel 
+	 * Chmod the public entry channel
 	 */
 	SYS_Chmod(PCSCLITE_CSOCK_NAME, S_IRWXO | S_IRWXG | S_IRWXU);
 
@@ -169,34 +169,29 @@ INTERNAL int SHMProcessEventsServer(PDWORD pdwClientID, int blocktime)
 {
 	fd_set read_fd;
 	int selret;
-	struct timeval tv;
-	
-	tv.tv_sec = 1;
-	tv.tv_usec = 0;
 
 	FD_ZERO(&read_fd);
 
 	/*
-	 * Set up the bit masks for select 
+	 * Set up the bit masks for select
 	 */
 	FD_SET(commonSocket, &read_fd);
 
 	selret = select(commonSocket + 1, &read_fd, (fd_set *) NULL,
-		(fd_set *) NULL, &tv);
+		(fd_set *) NULL, NULL);
 
 	if (selret < 0)
 	{
-		if ((!AraKiri) && (!ReCheckSerialReaders))
-			Log2(PCSC_LOG_CRITICAL, "Select returns with failure: %s",
-				strerror(errno));
+		if (EINTR == errno)
+			return -2;
+
+		Log2(PCSC_LOG_CRITICAL, "Select returns with failure: %s",
+			strerror(errno));
 		return -1;
 	}
 
-	if (selret == 0)
-		/* timeout */
-		return 2;
 	/*
-	 * A common pipe packet has arrived - it could be a new application  
+	 * A common pipe packet has arrived - it could be a new application
 	 */
 	if (FD_ISSET(commonSocket, &read_fd))
 	{
@@ -213,12 +208,12 @@ INTERNAL int SHMProcessEventsServer(PDWORD pdwClientID, int blocktime)
 			return 0;
 		}
 	}
-	
+
 	return -1;
 }
 
 /**
- * @brief 
+ * @brief
  *
  * Called by \c ContextThread().
  */
@@ -251,11 +246,11 @@ INTERNAL int SHMProcessEventsContext(PDWORD pdwClientID, psharedSegmentMsg msgSt
 	if (FD_ISSET(*pdwClientID, &read_fd))
 	{
 		/*
-		 * Return the current handle 
+		 * Return the current handle
 		 */
 		rv = SHMMessageReceive(msgStruct, sizeof(*msgStruct), *pdwClientID,
 				       PCSCLITE_SERVER_ATTEMPTS);
-		
+
 		if (rv == -1)
 		{	/* The client has died */
 			Log2(PCSC_LOG_DEBUG, "Client has disappeared: %d",
@@ -266,15 +261,15 @@ INTERNAL int SHMProcessEventsContext(PDWORD pdwClientID, psharedSegmentMsg msgSt
 
 			return 0;
 		}
-		
+
 		/*
-		 * Set the identifier handle 
+		 * Set the identifier handle
 		 */
 		Log2(PCSC_LOG_DEBUG, "correctly processed client: %d",
 			*pdwClientID);
 		return 1;
 	}
-	
+
 	return -1;
 }
 
