@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_svc.c 2128 2006-08-08 15:05:57Z rousseau $
+ * $Id: winscard_svc.c 2250 2006-11-30 22:26:35Z rousseau $
  */
 
 /**
@@ -31,6 +31,7 @@
 #include "winscard_svc.h"
 #include "sys_generic.h"
 #include "thread_generic.h"
+#include "readerfactory.h"
 
 /**
  * @brief Represents the an Application Context on the Server side.
@@ -90,18 +91,18 @@ LONG CreateContextThread(PDWORD pdwClientID)
 	if (i == PCSCLITE_MAX_APPLICATIONS_CONTEXTS)
 	{
 		SYS_CloseFile(psContext[i].dwClientID);
-		psContext[i].dwClientID = 0; 
+		psContext[i].dwClientID = 0;
 		Log2(PCSC_LOG_CRITICAL, "No more context available (max: %d)",
 			PCSCLITE_MAX_APPLICATIONS_CONTEXTS);
 		return SCARD_F_INTERNAL_ERROR;
 	}
-	
+
 	if (SYS_ThreadCreate(&psContext[i].pthThread, THREAD_ATTR_DETACHED,
 		(PCSCLITE_THREAD_FUNCTION( )) ContextThread,
 		(LPVOID) i) != 1)
 	{
 		SYS_CloseFile(psContext[i].dwClientID);
-		psContext[i].dwClientID = 0; 
+		psContext[i].dwClientID = 0;
 		Log1(PCSC_LOG_CRITICAL, "SYS_ThreadCreate failed");
 		return SCARD_E_NO_MEMORY;
 	}
@@ -111,7 +112,7 @@ LONG CreateContextThread(PDWORD pdwClientID)
 
 /*
  * A list of local functions used to keep track of clients and their
- * connections 
+ * connections
  */
 
 /**
@@ -119,7 +120,7 @@ LONG CreateContextThread(PDWORD pdwClientID)
  *
  * For each Client message a new instance of this thread is created.
  *
- * @param[in] dwIndex Index of an avaiable Application Context slot in 
+ * @param[in] dwIndex Index of an avaiable Application Context slot in
  * \c psContext.
  */
 static void ContextThread(LPVOID dwIndex)
@@ -130,7 +131,7 @@ static void ContextThread(LPVOID dwIndex)
 
 	Log2(PCSC_LOG_DEBUG, "Thread is started: %d",
 		psContext[dwContextIndex].dwClientID);
-	
+
 	while (1)
 	{
 		switch (rv = SHMProcessEventsContext(&psContext[dwContextIndex].dwClientID, &msgStruct, 0))
@@ -145,7 +146,7 @@ static void ContextThread(LPVOID dwIndex)
 					psContext[dwContextIndex].dwClientID);
 				MSGCleanupClient(dwContextIndex);
 				SYS_ThreadExit((LPVOID) NULL);
-			} 
+			}
 			break;
 
 		case 1:
@@ -158,7 +159,8 @@ static void ContextThread(LPVOID dwIndex)
 
 				/* the SCARD_TRANSMIT_EXTENDED anwser is already sent by
 				 * MSGFunctionDemarshall */
-				if (msgStruct.command != SCARD_TRANSMIT_EXTENDED)
+				if ((msgStruct.command != SCARD_TRANSMIT_EXTENDED)
+					&& (msgStruct.command != SCARD_CONTROL_EXTENDED))
 					rv = SHMMessageSend(&msgStruct, sizeof(msgStruct),
 						psContext[dwContextIndex].dwClientID,
 						PCSCLITE_SERVER_ATTEMPTS);
@@ -178,10 +180,25 @@ static void ContextThread(LPVOID dwIndex)
 						"Client is protocol version %d:%d",
 						veStr->major, veStr->minor);
 
+					veStr->rv = SCARD_S_SUCCESS;
+
+					/* client is newer than server */
+					if ((veStr->major > PROTOCOL_VERSION_MAJOR)
+						|| (veStr->major == PROTOCOL_VERSION_MAJOR
+							&& veStr->minor > PROTOCOL_VERSION_MINOR))
+					{
+						Log3(PCSC_LOG_CRITICAL,
+							"Client protocol is too new %d:%d",
+							veStr->major, veStr->minor);
+						Log3(PCSC_LOG_CRITICAL,
+							"Server protocol is %d:%d",
+							PROTOCOL_VERSION_MAJOR, PROTOCOL_VERSION_MINOR);
+						veStr->rv = SCARD_E_NO_SERVICE;
+					}
+
 					/* set the server protocol version */
 					veStr->major = PROTOCOL_VERSION_MAJOR;
 					veStr->minor = PROTOCOL_VERSION_MINOR;
-					veStr->rv = SCARD_S_SUCCESS;
 
 					/* send back the response */
 					rv = SHMMessageSend(&msgStruct, sizeof(msgStruct),
@@ -200,11 +217,11 @@ static void ContextThread(LPVOID dwIndex)
 			 * nothing else happens
 			 */
 			break;
-			
+
 		case -1:
 			Log1(PCSC_LOG_ERROR, "Error in SHMProcessEventsContext");
 			break;
-			
+
 		default:
 			Log2(PCSC_LOG_ERROR,
 				"SHMProcessEventsContext unknown retval: %d", rv);
@@ -220,13 +237,13 @@ static void ContextThread(LPVOID dwIndex)
  * cast the message data to the correct struct so that is can be demarshalled.
  * Then call the appropriate function to handle the request.
  *
- * Possible structs are: \c establish_struct \c release_struct 
+ * Possible structs are: \c establish_struct \c release_struct
  * \c connect_struct \c reconnect_struct \c disconnect_struct \c begin_struct
  * \c cancel_struct \c end_struct \c status_struct \c transmit_struct
  * \c control_struct \c getset_struct.
  *
  * @param[in] msgStruct Message to be demarshalled and executed.
- * @param[in] dwContextIndex 
+ * @param[in] dwContextIndex
  */
 LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
 {
@@ -245,7 +262,7 @@ LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
 	getset_struct *gsStr;
 
 	/*
-	 * Zero out everything 
+	 * Zero out everything
 	 */
 	rv = 0;
 	switch (msgStruct->command)
@@ -420,7 +437,7 @@ LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
 
 				rv = SHMMessageSend(pbRecvBuffer + PCSCLITE_MAX_MESSAGE_SIZE
 					- sizeof(*treStr),
-					treStr->size - PCSCLITE_MAX_MESSAGE_SIZE, 
+					treStr->size - PCSCLITE_MAX_MESSAGE_SIZE,
 					psContext[dwContextIndex].dwClientID,
 					PCSCLITE_SERVER_ATTEMPTS);
 				if (rv)
@@ -440,6 +457,75 @@ LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
 		}
 		break;
 
+	case SCARD_CONTROL_EXTENDED:
+		{
+			control_struct_extended *cteStr;
+			unsigned char pbSendBuffer[MAX_BUFFER_SIZE_EXTENDED];
+			unsigned char pbRecvBuffer[MAX_BUFFER_SIZE_EXTENDED];
+
+			cteStr = ((control_struct_extended *) msgStruct->data);
+			rv = MSGCheckHandleAssociation(cteStr->hCard, dwContextIndex);
+			if (rv != 0) return rv;
+
+			/* on more block to read? */
+			if (cteStr->size > PCSCLITE_MAX_MESSAGE_SIZE)
+			{
+				/* copy the first data part */
+				memcpy(pbSendBuffer, cteStr->data,
+					PCSCLITE_MAX_MESSAGE_SIZE-sizeof(*cteStr));
+
+				/* receive the second block */
+				rv = SHMMessageReceive(
+					pbSendBuffer+PCSCLITE_MAX_MESSAGE_SIZE-sizeof(*cteStr),
+					cteStr->size - PCSCLITE_MAX_MESSAGE_SIZE,
+					psContext[dwContextIndex].dwClientID,
+					PCSCLITE_SERVER_ATTEMPTS);
+				if (rv)
+					Log1(PCSC_LOG_CRITICAL, "reception failed");
+			}
+			else
+				memcpy(pbSendBuffer, cteStr->data, cteStr->cbSendLength);
+
+			cteStr->rv = SCardControl(cteStr->hCard, cteStr->dwControlCode,
+				pbSendBuffer, cteStr->cbSendLength,
+				pbRecvBuffer, cteStr->cbRecvLength,
+				&cteStr->pdwBytesReturned);
+
+			cteStr->size = sizeof(*cteStr) + cteStr->pdwBytesReturned;
+			if (cteStr->size > PCSCLITE_MAX_MESSAGE_SIZE)
+			{
+				/* two blocks */
+				memcpy(cteStr->data, pbRecvBuffer, PCSCLITE_MAX_MESSAGE_SIZE
+					- sizeof(*cteStr));
+
+				rv = SHMMessageSend(msgStruct, sizeof(*msgStruct),
+					psContext[dwContextIndex].dwClientID,
+					PCSCLITE_SERVER_ATTEMPTS);
+				if (rv)
+					Log1(PCSC_LOG_CRITICAL, "transmission failed");
+
+				rv = SHMMessageSend(pbRecvBuffer + PCSCLITE_MAX_MESSAGE_SIZE
+					- sizeof(*cteStr),
+					cteStr->size - PCSCLITE_MAX_MESSAGE_SIZE,
+					psContext[dwContextIndex].dwClientID,
+					PCSCLITE_SERVER_ATTEMPTS);
+				if (rv)
+					Log1(PCSC_LOG_CRITICAL, "transmission failed");
+			}
+			else
+			{
+				/* one block only */
+				memcpy(cteStr->data, pbRecvBuffer, cteStr->pdwBytesReturned);
+
+				rv = SHMMessageSend(msgStruct, sizeof(*msgStruct),
+					psContext[dwContextIndex].dwClientID,
+					PCSCLITE_SERVER_ATTEMPTS);
+				if (rv)
+					Log1(PCSC_LOG_CRITICAL, "transmission failed");
+			}
+		}
+		break;
+
 	default:
 		Log2(PCSC_LOG_CRITICAL, "Unknown command: %d", msgStruct->command);
 		return -1;
@@ -461,41 +547,61 @@ LONG MSGRemoveContext(SCARDCONTEXT hContext, DWORD dwContextIndex)
 
 	if (psContext[dwContextIndex].hContext == hContext)
 	{
-
 		for (i = 0; i < PCSCLITE_MAX_APPLICATION_CONTEXT_CHANNELS; i++)
 		{
 			/*
-			 * Disconnect each of these just in case 
+			 * Disconnect each of these just in case
 			 */
-			
+
 			if (psContext[dwContextIndex].hCard[i] != 0)
 			{
-				
+				PREADER_CONTEXT rContext = NULL;
+				DWORD dwLockId;
+
 				/*
-				 * We will use SCardStatus to see if the card has been 
-				 * reset there is no need to reset each time
-				 * Disconnect is called 
+				 * Unlock the sharing
 				 */
-				
-				rv = SCardStatus(psContext[dwContextIndex].hCard[i], 0, 0, 0, 0, 0, 0);
-				
-				if (rv == SCARD_W_RESET_CARD
-				    || rv == SCARD_W_REMOVED_CARD)
+				rv = RFReaderInfoById(psContext[dwContextIndex].hCard[i],
+					&rContext);
+				if (rv != SCARD_S_SUCCESS)
+					return rv;
+
+				dwLockId = rContext->dwLockId;
+				rContext->dwLockId = 0;
+
+				if (psContext[dwContextIndex].hCard[i] != dwLockId) 
 				{
-					SCardDisconnect(psContext[dwContextIndex].hCard[i], SCARD_LEAVE_CARD);
-				} else
-				{
-					SCardDisconnect(psContext[dwContextIndex].hCard[i], SCARD_RESET_CARD);
+					/*
+					 * if the card is locked by someone else we do not reset it
+					 * and simulate a card removal
+					 */
+					rv = SCARD_W_REMOVED_CARD;
 				}
+				else
+				{
+					/*
+					 * We will use SCardStatus to see if the card has been
+					 * reset there is no need to reset each time
+					 * Disconnect is called
+					 */
+					rv = SCardStatus(psContext[dwContextIndex].hCard[i], NULL,
+						NULL, NULL, NULL, NULL, NULL);
+				}
+
+				if (rv == SCARD_W_RESET_CARD || rv == SCARD_W_REMOVED_CARD)
+					SCardDisconnect(psContext[dwContextIndex].hCard[i],
+						SCARD_LEAVE_CARD);
+				else
+					SCardDisconnect(psContext[dwContextIndex].hCard[i],
+						SCARD_RESET_CARD);
 
 				psContext[dwContextIndex].hCard[i] = 0;
 			}
-
 		}
 
 		psContext[dwContextIndex].hContext = 0;
 		return SCARD_S_SUCCESS;
-	} 
+	}
 
 	return SCARD_E_INVALID_VALUE;
 }
@@ -506,9 +612,9 @@ LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard, DWORD dwContextIndex
 
 	if (psContext[dwContextIndex].hContext == hContext)
 	{
-		
+
 		/*
-		 * Find an empty spot to put the hCard value 
+		 * Find an empty spot to put the hCard value
 		 */
 		for (i = 0; i < PCSCLITE_MAX_APPLICATION_CONTEXT_CHANNELS; i++)
 		{
@@ -518,7 +624,7 @@ LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard, DWORD dwContextIndex
 				break;
 			}
 		}
-		
+
 		if (i == PCSCLITE_MAX_APPLICATION_CONTEXT_CHANNELS)
 		{
 			return SCARD_F_INTERNAL_ERROR;
@@ -528,7 +634,7 @@ LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard, DWORD dwContextIndex
 		}
 
 	}
-	
+
 	return SCARD_E_INVALID_VALUE;
 }
 
@@ -560,7 +666,7 @@ LONG MSGCheckHandleAssociation(SCARDHANDLE hCard, DWORD dwContextIndex)
 			return 0;
 		}
 	}
-	
+
 	/* Must be a rogue client, debug log and sleep a couple of seconds */
 	Log1(PCSC_LOG_ERROR, "Client failed to authenticate");
 	SYS_Sleep(2);
@@ -572,13 +678,13 @@ LONG MSGCleanupClient(DWORD dwContextIndex)
 {
 	if (psContext[dwContextIndex].hContext != 0)
 	{
-		SCardReleaseContext(psContext[dwContextIndex].hContext);	
+		SCardReleaseContext(psContext[dwContextIndex].hContext);
 		MSGRemoveContext(psContext[dwContextIndex].hContext, dwContextIndex);
 	}
 
 	psContext[dwContextIndex].dwClientID = 0;
 	psContext[dwContextIndex].protocol_major = 0;
 	psContext[dwContextIndex].protocol_minor = 0;
-	
+
 	return 0;
 }
